/* ----------------------------------------------------------
                            adc_demo.c

     Auslesen des ADC. Eingangsspannung wird ueber OP-
     Schaltung mit einem Offset versehen (siehe Schaltung),
     damit Spannung im Bereich von +-15V gemessen werden
     koennen.

     MCU     : atmega328p
     F_CPU   : 16000000

     21.11.2019    R. Seelig
   ---------------------------------------------------------- */

/*

               Schaltung der Eingangsbeschaltung

                                                  +U_ref

                                                    ^
                                                    |
                                                    |
                                                    |
                                                   ,-,
                                              R4   | |
           R1             |\                       '-'
           ___            |  \                      |
      o---|___|---+-------|+   \            R3      |
                  |       |      \          ___     |
      U_in        |       |       > --+----|___|----+----o
                  |       |      /    |
      o           |    +--|-   /      |                  U_out
      |          .-,   |  |  /        |
      |      R2  | |   |  |/          |                  o
      |          '-'   |              |                  |
      |           |    +--------------+                  |
      |           |                                      |
      |           |                                      |
     ---         ---                                    ---

     Uref= 3,3V

     R1= 1M
     R2= 33k
     R3= 1,2k
     R4= 4,7k

*/

#include <avr/io.h>
#include <util/delay.h>
#include <stdio.h>

#include "uart_all.h"
#include "my_printf.h"
#include "adc_single_mega.h"

#define printf    my_printf
#define delay     _delay_ms


#define plus15_digit      1005           // nach der Eingangsbeschaltung entspricht dieser ADC-Wert +15V
#define minus15_digit     320            // dto. fuer -15V

#define bin2spg(value)    ((value * 110.0) / 1023.0)



/* --------------------------------------------------------
   my_putchar

   wird von my-printf / printf aufgerufen und hier muss
   eine Zeichenausgabefunktion angegeben sein, auf das
   printf dann schreibt !
   -------------------------------------------------------- */
void my_putchar(char ch)
{
  uart_putchar(ch);
}

uint16_t bin2spg15(int16_t value)
{
  uint16_t digit_anz;
  float spgsteps;
  float r;

  digit_anz= plus15_digit - minus15_digit;
  spgsteps= 30.0 / digit_anz;                   // 30.0 = Spannweite -15V bis + 15V
  value -= minus15_digit;                       // value beinhaltet nun die Digits, die ueber -15V liegen
  r= (spgsteps * value);                        // r beinhaltet die Spannungsdifferenz zu -15V
  r = (r - 15.0) * 100;                         // * 100 um spaeter eine Festkommazahl ausgeben zu koennen
  return ((int16_t) r);
}

/* --------------------------------------------------------
                              main
   -------------------------------------------------------- */
int main(void)
{
  char     cnt;
  uint16_t mwert;
  uint8_t  value;

  uart_init();

  printf("\n\rAuslesen des ADC");
  printf("\n\r-------------------------------------------\n\n\r");

  _delay_us(30);
  adc_init(3,0);                                     // 1.1V  interne Referenzspannung (Mode 3), Input Channel 0

  while(1)
  {
    mwert= 0;
    for (cnt= 0; cnt< 10; cnt++)
    {
      mwert += getadc_10bit();
    }
    mwert= mwert / 10;

    delay(300);
    printfkomma= 2;
    printf("\r ADC-Value Mittelwert Digits= %d   ; Voltage = %k    ; Voltage15 = %k    ",
             mwert, (uint16_t)bin2spg(mwert), (int16_t)bin2spg15(mwert));
  }
}

