/* ----------------------------------------------------------
     analog.c

     Ausgabe analoger Spannungen mittels Pulsweiten-
     modulation (PWM) an PB1

     Hinweis: Da der ATmega keinen DAC-Wandler hat, kann er
              logischerweise nur mittlere Spannungswerte, die
              ueber ein Puls-Pausenverhaeltnis erreich
              werden generieren. Werden Gleichspannungen be-
              noetigt, so ist ein Tiefpassfilter nachzu-
              schalten.

     MCU     : atmega328p
     F_CPU   : 16000000

     07.06.2018    R. Seelig
   ---------------------------------------------------------- */

#include <util/delay.h>
#include <avr/io.h>
#include <math.h>

#include "avr_gpio.h"

#define pwmdir      DDRB
#define pwmport     PORTB
#define pwmout      PB1                       //  PB1 / OC1A


#define uein        4.98                      // Spannungspegel, den ein logischer 1 Pegel
                                              // besitzt (gemessen)

/* -------------------------------------------------
                      pwm_init

    setzt den PWM - Generator als Fast PWM
  ------------------------------------------------- */
void pwm_init()
{
   pwmdir |= (1<<pwmout);                // OC1A .. PB1 als Ausgang

   // WGM10= 0 ; WGM11 WGM12, WGM13 = 1 ===> Modus 14 (Fast PWM)
   // Prescaler wird durch CS10, CS11, CS12 (1.0.0) gesetzt (keine Taktvorteilung)

   TCCR1A = (1<<COM1A1) | (1<<WGM11);
   TCCR1B = (1<<WGM12) | (1<<WGM13) | (1<<CS10);
}

/* -------------------------------------------------
                      pwm_set

    setzt die Gesamtdauer und Pulsdauer der PWM
    Uebergabe:
       tim1 ==> Gesamtdauer
       tim2 ==> Pulsdauer
  ------------------------------------------------- */
void setpwm(uint16_t tim1, uint16_t tim2)
{

  ICR1H = (tim1>>8);
  ICR1L = (uint8_t) tim1 & 0x00ff;
  OCR1AH = (tim2>>8);
  OCR1AL = (uint8_t) tim2 & 0x00ff;
}

/* -------------------------------------------------
                      analogout

     ist an den PWM-Anschluss ein Tiefpass ange-
     schlossen, so wird der in value angegebene
     Spannungswert eingestellt.

     Uebergabe:
       value  : auszugebender Spannungswert
       refwert: Spannung, die ein logischer 1
                Pegel besitzt
   ------------------------------------------------- */
void analogout(float value, float refwert)
{
  uint16_t regwert;
  uint16_t regout;
  float    r;

  r= (value * 1023) / refwert;
  regout= r;

  setpwm(1023,regout);

}


/* ---------------------------------------------------------------------------
                                    M A I N
   --------------------------------------------------------------------------- */
int main(void)
{
  char     b;
  float    uout;

  pwm_init();

  analogout(1.2, uein);

  while(1);
}
