/* -----------------------------------------------
                   blink_n_tast.c

     eine blinkende Leuchtdiode ist das "Hallo
     Welt" Programm in der Microcontrollerwelt.

     Hier in der "Luxusvariante" mit einem Taster.

     Die "Tasterklicks" werden gezaehlt und geht
     nach einer gewissen Zeit kein Tasterklick
     ein, blinkt die LED in der Anzahl der ein-
     gegangenen Tastenklicks.

     MCU            : ATmegaxx8
     Hardware       : Taster, LED
     Takt           : 16 MHz extern

     Compiler       : AVR-GCC 4.9.2
     IDE            : keine (Makefile)

     22.12.2017     R. Seelig

   ----------------------------------------------- */


#include <avr/io.h>
#include <util/delay.h>

#include "avr_gpio.h"

// Zuordnung LED und Taster ( Button )
// -----------------------------------------------
#define led_init()           PB5_output_init()
#define led_on()             PB5_clr()                // LED ist gegen +Ub
#define led_off()            PB5_set()                // geschaltet

#define button_init()        PC3_input_init()
#define is_button()          ( !is_PC3() )            // "negierte" Logik, Taster
                                                      // schaltet gegen GND

// Timing
// -----------------------------------------------

#define blkspeed             300
#define but_timeout          500
#define prelltime             40                      // abhaengig von der Qualitaet des
                                                      // Tasters

/* -----------------------------------------------
                        delay

     eigenes Delay, damit die Wartezeit mittels
     einer Variable uebergeben werden kann
   ----------------------------------------------- */
void delay(uint16_t ms)
{
  int i;
  for (i= 0; i < ms; i++) _delay_ms(1);
}

/* -----------------------------------------------
                      blink_repeat

     laesst die Leuchtdiode in der Haeufigkeit
     anz blinken.
   ----------------------------------------------- */
void blink_repeat(uint16_t anz)
{
  int i;
  for (i= 0; i < anz; i++)
  {
    led_on();
    _delay_ms(blkspeed);
    led_off();
    _delay_ms(blkspeed);
  }
}

/* -----------------------------------------------
                 button_getklicks

     wartet auf einen ersten Tastendruck und
     zaehlt anschliessend die Anzahl der Klicks.

     Erfolgt nach Ablauf der Timeoutzeit (in ms)
     kein weiterer Tastendruck, wird die Anzahl
     der Klicks zurueckgegeben.
   ----------------------------------------------- */
uint8_t button_getklicks(uint16_t timeout)
{
  uint16_t timeout_cnt = 0;
  uint8_t  klicks = 0;

  while(!is_button());              // warten auf ersten Tastendruck
  delay(prelltime);                 // Taste entprellen
  while(is_button());               // warten auf loslassen
  delay(prelltime);                 // Taste entprellen
  klicks++;

  while(timeout_cnt < timeout)      // solange Tmeout nicht erreicht
                                    // ist, werden Tastenklicks gezaehlt
  {
    if (is_button())
    {
      delay(prelltime);
      while(is_button());           // warten bis Taster wieder losgelassen
      delay(prelltime);

      timeout_cnt= 0;               // Timeoutcounter zuruecksetzen
      klicks++;
    }
    delay(1);
    timeout_cnt++;
  }
  return klicks;
}


/* ---------------------------------------------------------------------
                                 M A I N
   --------------------------------------------------------------------- */
int main(void)
{
  uint8_t klicks;

  led_init();
  button_init();

  blink_repeat(2);                  // zweimal Blinken um Bereitschaft
                                    // anzuzeigen
  while (1)                         // fuer immer blinken
  {
    klicks= button_getklicks(but_timeout);
    blink_repeat(klicks);
  }
}
