/* -----------------------------------------------
                   blink_n_tast_int.c

     eine blinkende Leuchtdiode ist das "Hallo
     Welt" Programm in der Microcontrollerwelt.

     Hier in der "Luxusvariante" mit einem Taster
     und mittels Timerinterrupt realisiert: Das
     Druecken der Taste wird gezaehlt, wird
     "laengere" Zeit keine Taste gedrueckt, blinkt
     die LED genau so oft, wie die Taste gedrueckt
     wurde.

     MCU            : ATmegaxx8
     Hardware       : Taster, LED
     Takt           : 16 MHz extern

     Compiler       : AVR-GCC 4.9.2
     IDE            : keine (Makefile)

     22.12.2017     R. Seelig

   ----------------------------------------------- */


#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#include "avr_gpio.h"


// Zuordnung LED und Taster ( Button )
// -----------------------------------------------
#define led_init()           PB5_output_init()
#define led_on()             PB5_clr()                // LED ist gegen +Ub
#define led_off()            PB5_set()                // geschaltet

#define button_init()        PC3_input_init()
#define is_button()          ( !is_PC3() )            // "negierte" Logik, Taster
                                                      // schaltet gegen GND

// Timing
// -----------------------------------------------

#define blkspeed             250
#define but_timeout          500
#define prelltime             40                      // abhaengig von der Qualitaet des
                                                      // Tasters


// Globale Variable als Systemticker
// -----------------------------------------------
volatile uint16_t systick= 0;                         // wird vom Timerinterrupt jede ms
                                                      // hochgezaehlt


/* ---------------------------------------------------------------------
                             TIMER1 Interrupt

                      wird jede Millisekunde aufgerufen
   --------------------------------------------------------------------- */
ISR (TIMER1_COMPA_vect)
{
  static uint16_t  blk_timer= 0, prell_timer= 0;
  static uint16_t  timeout_cnt= 0;
  static uint8_t   but_stat= 0, but_oldstat= 0;
  static uint8_t   but_free= 1;
  static uint8_t   klicks= 0;
  static uint8_t   blk_todo = 0;

  systick++;

  // Anzahl Blinken abarbeiten
  if (blk_todo)
  {
     blk_timer++;
     if (blk_timer== blkspeed) led_on();
     if (blk_timer>= (blkspeed * 2))
     {
       led_off(); blk_timer= 0;
       blk_todo--;
     }
  }

  else
  // Wenn Blinken nicht aktiv ist, kann Taste abgefragt werden
  {
    if (but_free)                      // Button ist nicht gesperrt
    {
      but_stat= is_button();

      if (but_stat && !but_oldstat)    // Button wurde gedrueckt
      {
        but_free= 0;                   // Button fuer Entprelldauer gesperrt
        prell_timer= 0;
        but_oldstat= but_stat;
      }

      if (!but_stat && but_oldstat)    // Button wurde losgelassen
      {
        timeout_cnt= 0;
        but_free= 0;                   // Button fuer Entprelldauer gesperrt
        prell_timer= 0;
        but_oldstat= but_stat;
        klicks++;                      // Anzahl der Klicks zaehlen
      }

    }

    else                               // Button ist fuer die Entprelldauerzeit gesperrt
    // Entprellzeit herunterzaehlen
    {
      prell_timer++;
      if (prell_timer== prelltime)
      {
        but_free= 1;
        prell_timer= 0;
      }
    }
  }
  timeout_cnt++;
  if ( (timeout_cnt== but_timeout) && klicks)
  {
    blk_todo= klicks;
    klicks= 0;
  }
}


/* ---------------------------------------------------------------------
                               timer1_init

      initialisiert den Timer fuer ein Ausloesen des Interrupts nach
      jeder Millisekunde
   --------------------------------------------------------------------- */
void timer1_init(void)
//  (F_CPU / t1reload)-1  => Wert Vergleichsregister. Erreicht der Timer diesen Wert,
//  wird ein Interrupt ausgeloest. Die Zeit die vergeht bis zum Erreichen des Vergleich-
//  wertes: ((F_CPU / t1reload)-1) * Zykluszeit
#define t1reload 1000L                                // Interruptaufruf jede Millisekund

{
  TCCR1B = 1<<WGM12 | 1<<CS10;
  OCR1A = F_CPU / t1reload-1;
  TCNT1 = 0;

  TIMSK1 = 1<<OCIE1A;
  sei();
}


/* ---------------------------------------------------------------------
                                 M A I N
   --------------------------------------------------------------------- */
int main(void)
{
  led_init();
  button_init();
  timer1_init();

  led_off();

  while (1);                         // Taste zaehlen und blinken
                                     // geschieht im Interrupt
}
