/* --------------------------------------------------------
                      ds18b20_demo.c

     rudimentaere Routinen um einen DS18B20 Temperatursensor
     einzulesen, eine Software-Uhr zu implementieren
     und Werte ueber serielle Schnittstelle ein- und ausgeb-
     bar zu machen

     MCU : 328p
     Takt: 16 MHz

     02.07.2024   R. Seelig
   -------------------------------------------------------- */
#include <string.h>

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/io.h>
#include <util/delay.h>
#include <avr/pgmspace.h>

#include "./avr_gpio.h"
#include "./ds18b20.h"
#include "./tftmono.h"
#include "./my_printf32.h"
#include "./my_time.h"
#include "./uart_all.h"

#define printf      my_printf
#define delay       _delay_ms


mydate_time        mydate;
volatile uint32_t  sekcount = 0;
volatile uint32_t  oldsek;


// Die Ganggenauigkeit der Uhr ist vom Quarztakt des Controllers abhaengig.
// Da diese nicht hinsichtlich der Uhr nicht sehr genau sind, kann hier ein
// Korrekturfaktor angegeben werden, der jedes mal um 00:01.30 Uhr vorgenommen
// werden kann. D.h. hier kann der Sekundenwert eingegeben, um den die Software-
// uhr pro 24 Stunden falsch laeuft.

#define sek_calib  1

#if (sek_calib == 1)
  #define sek_korrektur   -10     // Anzahl Sekunden um die die Uhr alle 24h korrigiert wird
                                  // max. Wert +-29 Sekunden
  uint8_t dosekcalib = 1;
#endif

enum OUT_CHANNEL { lcd, uart };
uint8_t outstream = uart;

/* --------------------------------------------------
                     dez2bcd

      wandelt eine dezimale Zahl in eine BCD
      Bsp: value = 45
      Rueckgabe    0x45
   -------------------------------------------------- */
uint8_t dez2bcd(uint8_t value)
{
  uint8_t hiz,loz,c;

  hiz= value / 10;
  loz= (value -(hiz*10));
  c= (hiz << 4) | loz;
  return c;
}

/* --------------------------------------------------------
   my_putchar

   wird von my-printf / printf aufgerufen und hier muss
   eine Zeichenausgabefunktion angegeben sein, auf das
   printf dann schreibt !
   -------------------------------------------------------- */
void my_putchar(char ch)
{
  if (outstream== uart) uart_putchar(ch); else lcd_putchar(ch);
}


/* --------------------------------------------------------
                           timer1_init

     initialisiert Timer1 Compare Interrupt Modus fuer
     Aufruf des Interrupts jede Sekunde

   -------------------------------------------------------- */
void timer1_init(void)
{
  /* ---------------------------------------------
     CS10 / CS11 / CS12 sind Steuerbits
     des Taktvorteilers.

     CS 12 | CS11 | CS10
     ----------------------------------------------
       0      0      0   kein Takt, Timer gestoppt
       0      0      1   clk / 1 (kein Teiler)
       0      1      0   clk / 8
       0      1      1   clk / 64
       1      0      0   clk / 256
       1      0      1   clk / 1024
     ---------------------------------------------- */
  TCCR1B = 1 << WGM12 | 1 << CS10 | 1 << CS12;       // Taktvorteiler = 1024
  OCR1A = F_CPU / 1024;                              // 16 MHz / 1024 = 15625
                                                     // nach 15625 Timertakten wird Interrupt ausgeloest
  TCNT1 = 0;

  TIMSK1 = 1 << OCIE1A;
  sei();
}

/* --------------------------------------------------------
     Timer 1 compare A Interruptroutine

     wird jede Sekunde aufgerufen
   -------------------------------------------------------- */
ISR (TIMER1_COMPA_vect)
{
  volatile uint8_t  i;
  volatile uint32_t mw;

  sekcount++;
  mydate= mydate_get(sekcount);

  #if (sek_calib == 1)

    // es erfolgt eine Zeitkorrektur um 00:01.30 Uhr
    if ((mydate.hour== 0) && (mydate.min== 1) && (mydate.sec== 30) && dosekcalib)
    {
      sekcount += sek_korrektur;
      dosekcalib= 0;
    }

    if ((mydate.hour== 0) && (mydate.min== 2) && (mydate.sec== 0) && !dosekcalib)
    {
      dosekcalib= 1;
    }

  #endif

}

/* --------------------------------------------------
     uhr_stellen

     die Softwareuhr benutzerabgefragt stellen
   -------------------------------------------------- */
void uhr_stellen(void)
{
  int       i;

  cli();                     // Uhr anhalten, damit Eingaben nicht ueberschrieben werden
  do
  {
    printf("\n\rTag    : ");
    i= uart_readint();
  } while ( !((i > -1) && (i < 32)));
  mydate.day= i;

  do
  {
    printf("\n\rMonat  : ");
    i= uart_readint();
  } while ( !((i > -1) && (i < 13)));
  mydate.month= i;

  do
  {
    printf("\n\rJahr   : ");
    i= uart_readint();
  } while ( !((i > -1) && (i < 100)));
  mydate.year= 2000+i;

  do
  {
    printf("\n\rStunde : ");
    i= uart_readint();
  } while ( !((i > -1) && (i < 24)));
  mydate.hour= i;

  do
  {
    printf("\n\rMinute : ");
    i= uart_readint();
  } while ( !((i > -1) && (i < 60)));
  mydate.min= i;

  do
  {
    printf("\n\rSekunde: ");
    i= uart_readint();
  } while ( !((i > -1) && (i < 60)));

  mydate.sec= i;

  sekcount= mydate_getstamp(mydate);      // Zeitstempel erzeugen
  sei();
}                                         // Uhr freigeben


/* --------------------------------------------------------
                             main
   -------------------------------------------------------- */
int main(void)
{
  uint8_t  ch;
  uint16_t i;
  int      wt;
  uint16_t intervall;
  int16_t  temp;

  char wtag[][3] = {"So","Mo","Di","Mi","Do","Fr","Sa"};


  timer1_init();
  oldsek= sekcount;
  uart_init();
  lcd_init();
  lcd_enable();

  clrscr();
  setfont(fnt5x7);
  outstream= lcd;
  textsize= 0;
  temp= 312;
  printf("Uhrzeit ueber\n\rUART stellen");

  outstream= uart;
  printf("\n\r------------------------------------");
  printf("\n\r        DS18B20 Demo");
  printf("\n\r------------------------------------\n\n\r");


  // zu Testzwecken
  cli();
  mydate.year= 2024;
  mydate.month= 7;
  mydate.day= 2;
  mydate.hour= 11;
  mydate.min= 2;
  mydate.sec= 27;
  sekcount = mydate_getstamp(mydate);
  sei();

//  uhr_stellen();

  printf("\n\n\r");

  clrscr();
  printfkomma= 1;
  while(1)
  {
    while(oldsek== sekcount);

    temp= ds18b20_gettemp();

    oldsek= sekcount;
    wt= mydate_getwtag(mydate);

    outstream= uart;
    // Umrechnung in BCD-Zahlen und Darstellung als Hexwerte, damit
    // eine fuehrende Null mit ausgegeben wird (ist den eingeschraenkten
    // Moeglichkeiten von - my_printf - geschuldet )
    printf("   %s:  %x.%x.%d  %x.%x:%x  ", wtag[wt],                                  \
               dez2bcd(mydate.day), dez2bcd(mydate.month), mydate.year,               \
               dez2bcd(mydate.hour), dez2bcd(mydate.min), dez2bcd(mydate.sec));
    printf(" Temp.: %k oC \r",temp);

    outstream= lcd;
    textsize= 0;
    setfont(fnt5x7);
    gotoxy(0,0);
    printf("%s: %x.%x.%d", wtag[wt], dez2bcd(mydate.day), dez2bcd(mydate.month), mydate.year);
    gotoxy(3,1);
    printf("%x.%x:%x", dez2bcd(mydate.hour), dez2bcd(mydate.min), dez2bcd(mydate.sec));
    setfont(fnt8x8);
    textsize= 2;
    gotoxy(0,3);
    printf("%k%c",temp,130);       // umdefiniertes Ascii-Zeichen 130 fuer hochgestelltes o

  }
}
