/* -------------------------------------------------------------
                        my_time_demo.c

    Demoprogramm fuer my_time

    Hardware  : alle AVR mit genuegendem Flash-Speicher
    IDE       : keine (Editor / make)

    Toolchain : avr-gcc

    30.06.2024   R. Seelig
   ------------------------------------------------------------- */

#include <stdint.h>
#include <avr/interrupt.h>

#include "./my_printf32.h"
#include "./my_time.h"
#include "./uart_all.h"

#define printf    my_printf
#define delay     _delay_ms

volatile uint32_t sekcount;                   // Sekundenzaehler

/* --------------------------------------------------------
   my_putchar

   wird von my-printf / printf aufgerufen und hier muss
   eine Zeichenausgabefunktion angegeben sein, auf das
   printf dann schreibt !
   -------------------------------------------------------- */
void my_putchar(char ch)
{
  uart_putchar(ch);
}

/* --------------------------------------------------
                     dez2bcd

      wandelt eine dezimale Zahl in eine BCD
      Bsp: value = 45
      Rueckgabe    0x45
   -------------------------------------------------- */
uint8_t dez2bcd(uint8_t value)
{
  uint8_t hiz,loz,c;

  hiz= value / 10;
  loz= (value -(hiz*10));
  c= (hiz << 4) | loz;
  return c;
}


/* --------------------------------------------------------
                     Interruptvector Timer1

     uebernimmt die Frequenzzaehlervariable in die Variable
     die angezeigt wird und setzt die Zaehlervariable
     zurueck
   -------------------------------------------------------- */
ISR (TIMER1_COMPA_vect)
{
  sekcount++;
}

/* --------------------------------------------------------
                           timer1_init

     initialisiert Timer1 Compare Interrupt Modus fuer
     Aufruf des Interrupts jede Sekunde

   -------------------------------------------------------- */
void timer1_init(void)
{
  /* ---------------------------------------------
     CS10 / CS11 / CS12 sind Steuerbits
     des Taktvorteilers.

     CS 12 | CS11 | CS10
     ----------------------------------------------
       0      0      0   kein Takt, Timer gestoppt
       0      0      1   clk / 1 (kein Teiler)
       0      1      0   clk / 8
       0      1      1   clk / 64
       1      0      0   clk / 256
       1      0      1   clk / 1024
     ---------------------------------------------- */
  TCCR1B = 1 << WGM12 | 1 << CS10 | 1 << CS12;       // Taktvorteiler = 1024
  OCR1A = F_CPU / 1024;
  TCNT1 = 0;

  TIMSK1 = 1 << OCIE1A;
  sei();
}


/* -------------------------------------------------------------
                                main
   ------------------------------------------------------------- */
int main(void)
{

  char wtag[][3] = {"So","Mo","Di","Mi","Do","Fr","Sa"};

  mydate_time   mydate;
  uint32_t      timestamp;
  int           wt;

  volatile uint32_t  oldsek;     // !!! volatile notwendig, damit nicht wegoptimiert wird

  uart_init();

  printf("\n\n\rmy_time fuer ATmegaxx8\n\n\r");
  printf("Protokoll 8N1 %d0 Bd\n\r", BAUDRATE / 10);
  printf("------------------------------------\n\r");

  timestamp= 1719862473ul;          // 01.07.2024   19:34.33
  mydate= mydate_get(timestamp);    // ein Datum aus Zeitstempel berechnen

  wt= mydate_getwtag(mydate);

  // Umrechnung in BCD-Zahlen und Darstellung als Hexwerte, damit
  // eine fuehrende Null mit ausgegeben wird (ist den eingeschraenkten
  // Moeglichkeiten von - my_printf32 - geschuldet )
  printf("\n\r  %s:  %x.%x.%d  %x.%x:%x  ", wtag[wt],                                  \
             dez2bcd(mydate.day), dez2bcd(mydate.month), mydate.year,               \
             dez2bcd(mydate.hour), dez2bcd(mydate.min), dez2bcd(mydate.sec));

  // ein Datum vorgeben
  mydate.year= 2020;
  mydate.month= 2;
  mydate.day= 29;
  mydate.hour= 23;
  mydate.min= 12;
  mydate.sec= 17;

  timestamp= mydate_getstamp(mydate);
  mydate= mydate_get(timestamp);         // zur Ueberpruefung, dass Datum korrekt in Zeitstempel und wieder
                                         // zurueck gerechnet wird

  wt= mydate_getwtag(mydate);
  printf("\n\n\rZeitstempel: %D", timestamp);
  printf("\n\r  %s:  %x.%x.%d  %x.%x:%x  ", wtag[wt],                                  \
             dez2bcd(mydate.day), dez2bcd(mydate.month), mydate.year,                   \
             dez2bcd(mydate.hour), dez2bcd(mydate.min), dez2bcd(mydate.sec));

  // Datum fuer eine Stunde spaeter
  timestamp += sec_per_hour;
  mydate= mydate_get(timestamp);
  wt= mydate_getwtag(mydate);
  printf("\n\n\rZeitstempel eine Stunde spaeter %D", timestamp);
  printf("\n\r  %s:  %x.%x.%d  %x.%x:%x  ", wtag[wt],                                  \
             dez2bcd(mydate.day), dez2bcd(mydate.month), mydate.year,                   \
             dez2bcd(mydate.hour), dez2bcd(mydate.min), dez2bcd(mydate.sec));

  printf("\n\n\r");

  sekcount= 1719862473ul;      // 01.07.2024   19:34.33
  mydate= mydate_get(sekcount);
  oldsek= sekcount;
  timer1_init();
  while(1)
  {
    while (oldsek == sekcount);
    oldsek= sekcount;

    mydate= mydate_get(sekcount);
    wt= mydate_getwtag(mydate);
    printf("  %s:  %x.%x.%d  %x.%x:%x  \r", wtag[wt],                                  \
               dez2bcd(mydate.day), dez2bcd(mydate.month), mydate.year,                     \
               dez2bcd(mydate.hour), dez2bcd(mydate.min), dez2bcd(mydate.sec));
  }
}

