/* -------------------------------------------------
                     fcount.c

   einfacher Frequenzzaehler fuer ATMega mit Aus-
   gabe auf N5110 Display.

   Frequenzeingang (TTL-Pegel): PD3
   Pinbelegung SPI-Display siehe tftmono.h

   MCU       : ATmega88/168/328p
   IDE       : keine (Editor / make)
   Toolchain : AVR-GCC

   23.09.19 by R. Seelig
  -------------------------------------------------- */

#include <avr/io.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>

#include "avr_gpio.h"

#include "tftmono.h"
#include "my_printf32.h"

#define  delay                _delay_ms
#define  printf               my_printf

#define cinput_init()         PD3_input_init()
#define is_cinput()           is_PD3()


  // ------------------------------------------------
  //       Registernamen fuer ATmega8 anpassen
  // ------------------------------------------------

  #if defined (__AVR_ATmega8__)
    #define TIMSK1     TIMSK
    #define EICRA      MCUCR
    #define EIMSK      GICR
  #endif



volatile uint32_t  fc;
volatile uint32_t  freq;
volatile uint8_t   timflag;                                                               // wird durch Interrupt gesetzt


/* --------------------------------------------------------
                           int1_init

     initialisiert externer Interrrupt 1 fuer Interrupt-
     aufruf bei jeder Zustandsaenderung (2 Aufrufe per
     Periode)

     INT1 - Pin ist PD3
   -------------------------------------------------------- */
void int1_init(void)
{
  EICRA |= (1 << ISC11);
  EIMSK |= (1 << INT1);
  sei();
}

/* --------------------------------------------------------
                   Interruptvector ex. INT1

     zaehlt die Frequenzzaehlervariable hoch
   -------------------------------------------------------- */

ISR (INT1_vect)
{
  fc++;
}

/* --------------------------------------------------------
                     Interruptvector Timer1

     uebernimmt die Frequenzzaehlervariable in die Variable
     die angezeigt wird und setzt die Zaehlervariable
     zurueck
   -------------------------------------------------------- */
ISR (TIMER1_COMPA_vect)
{
  freq= fc;
  fc= 0;
}

/* --------------------------------------------------------
                           timer1_init

     initialisiert Timer1 Compare Interrupt Modus fuer
     Aufruf des Interrupts jede Sekunde

   -------------------------------------------------------- */
void timer1_init(void)
{
  /* ---------------------------------------------
     CS10 / CS11 / CS12 sind Steuerbits
     des Taktvorteilers.

     CS 12 | CS11 | CS10
     ----------------------------------------------
       0      0      0   kein Takt, Timer gestoppt
       0      0      1   clk / 1 (kein Teiler)
       0      1      0   clk / 8
       0      1      1   clk / 64
       1      0      0   clk / 256
       1      0      1   clk / 1024
     ---------------------------------------------- */
  TCCR1B = 1 << WGM12 | 1 << CS10 | 1 << CS12;       // Taktvorteiler = 1024
  OCR1A = F_CPU / 1024;
  TCNT1 = 0;

  TIMSK1 = 1 << OCIE1A;
  sei();
}


/* --------------------------------------------------------
   putchar

   wird von my-printf / printf aufgerufen und hier muss
   eine Zeichenausgabefunktion angegeben sein, auf das
   printf dann schreibt !
   -------------------------------------------------------- */
void my_putchar(char ch)
{
  lcd_putchar(ch);
}


/* --------------------------------------------------------
                        getziffs

     gibt die Anzahl der Ziffern eines 32-Bit Integers
     zurueck
   -------------------------------------------------------- */
uint8_t getziffs(uint32_t z)
{
  uint8_t anz = 0;

  do
  {
    z= z / 10;
    anz++;
  } while (z);
  return anz;
}


/* ------------------------------------------------------------------------
                                    MAIN
   ------------------------------------------------------------------------ */

int main(void)
{
  uint8_t i,l;


  lcd_init();
  cinput_init();
  int1_init();
  timer1_init();
  clrscr();

  setfont(fnt5x7);
  textsize= 0;
  gotoxy(0,1);

  printf("200kHz Counter");
  gotoxy(0,2);
  printf("--------------");
  fc= 0;
  setfont(fnt5x7);
  gotoxy(10,4);
  printf("Hz");
  setfont(fnt8x8);
  while(1)
  {
    gotoxy(1,4);
    for (i=0; i< 6- getziffs(freq); i++) my_putchar(' ');
    printf("%l",freq);

    delay(50);
  }
}
