/* -------------------------------------------------------
                        glcd_spi_spiro.c

     Grafik-Demoprogramm fur SPI LCD 128x128 Pixel

     MCU   :  ATmega88/168/328p
     Takt  :  16 MHz

     21.01.2020  R. Seelig
   ------------------------------------------------------ */

/* -------------------------------------------------------
     In tftdisplay.h wird angegeben, wie das Display
     angeschlossen werden muss..
     Aktuell ist dieses:

                     #define pindefs     1

     in tft_pindefs.h wird dieses ausgewertet und fuer
     die Pindefinition 1 gilt folgende Anschlussbelegung:

     Die Anschlussnamen der Displays variieren fuer ein
     und dieselbe Bedeutung:
                     SDI - SDA
                     CLK - SCK
                     CS  - CE
                     A0  - DC - D/C - RS
                     rst - reset

     Display    Arduino    Atmega    Pinnr.(28 pol.)
     -----------------------------------------------
       sck        D13       PB5          19
       sda        D11       PB3          17
       cs         D10       PB2          16
       DC         D9        PB1          15
       rst        D8        PB0          14

   ------------------------------------------------------- */

#include <util/delay.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <stdlib.h>
#include <string.h>

#include "tftdisplay.h"
#include "my_printf.h"

#define printf           tiny_printf
#define delay            _delay_ms

/* --------------------------------------------------------
   my_putchar

   wird von my-printf / printf aufgerufen und hier muss
   eine Zeichenausgabefunktion angegeben sein, auf das
   printf dann schreibt !
   -------------------------------------------------------- */
void my_putchar(char ch)
{
  lcd_putchar(ch);
}

/* --------------------------------------------------------
                      spiro_generate

       zeichnet einee Spirographen auf das Display
   -------------------------------------------------------- */
void spiro_generate(int inner, int outer, int evol, int resol, uint16_t col)
{
  const int c_width  = 128;
  const int c_height = 128;
  float     inner_xpos, inner_ypos;
  float     outer_xpos, outer_ypos;
  float     j, k;
  int       i;

  inner_xpos = (c_width / 2.0f);
  inner_ypos = (c_height / 2.0f) + inner;

  outer_xpos= inner_xpos;
  outer_ypos= inner_ypos + outer;
  turtle_moveto(outer_xpos, outer_ypos);

  for (i= 0; i< resol + 1; i++)
  {
    j= ((float)i / resol) * (2.0f * M_PI);
    inner_xpos = (c_width / 2.0f) + (inner * sin(j));
    inner_ypos = (c_height / 2.0f) + (inner * cos(j));

    k= j * ((float)evol / 10.0f);

    outer_xpos= inner_xpos + (outer * sin(k));
    outer_ypos= inner_ypos + (outer * cos(k));

    turtle_lineto(outer_xpos, outer_ypos, col);
    delay(5);
  }
}

/* --------------------------------------------------------
   -------------------------------------------------------- */
int main(void)
{
  uint16_t x;
  uint16_t coltmp;

  lcd_init();
  lcd_enable();

  lcd_orientation(0);           // Ausgabeposition auf dem Display

  bkcolor= rgbfromega(0);
  textcolor= rgbfromvalue(0x00,0x80,0x80);
  clrscr();

  // ein paar Linien Zeichnen
  for (x= 0; x< 125; x+= 5)
  {
    line(0, 127, x, 0, rgbfromega(blue));
    delay(20);
  }
  for (x= 0; x< 125; x+= 5)
  {
    line(127, 127, x, 0, rgbfromega(grey));
    delay(20);
  }

  coltmp= rgbfromvalue(0x10, 0x10, 0x10);
  fillrect(18,10,112,46, coltmp);
  bkcolor= coltmp;
  gotoxy(3,2); printf("Spirograph");
  gotoxy(3,4); printf("mit ATmega");
  delay(2000);
  bkcolor= 0;
  clrscr();

  while(1)
  {
  spiro_generate(21, 21, 80, 220, rgbfromvalue(50, 255, 50));
  spiro_generate(32, 32, 140,220, rgbfromega(1));
  delay(5000);

  spiro_generate(32, 32, 140,220, rgbfromvalue(0x00,0x20,0x00));;
  spiro_generate(21, 21, 80, 220, rgbfromvalue(0x00,0xcf,0x00));
  delay(5000);

  spiro_generate(32, 32, 140,220, rgbfromega(4));
  spiro_generate(21, 21, 80, 220, rgbfromvalue(0xff,0x80,0x00));
  delay(5000);
  }
  gotoxy(0,0);
  printf("Spirograph");

}

