/*   -------------------------------------------------------
                          hd44780.h

     Header fuer grundlegende Funktionen eines HD44780
     kompatiblen LC-Textdisplays

     Hardware : Text-LCD

     MCU      :  AVR

     14.02.2020  R. Seelig

     Hinweis:
     Alle Anschlusspins des LCD muessen mit einem 1k
     Pop-Up Widerstand gegen +5V geschaltet sein, weil ein
     Hi-Pegel auf den Leitungen durch ein Konfigurieren
     der Controllerpins als Eingaenge erreicht wird.

     Der Vorteil dieser Methode besteht darin, dass ein
     Controller somit auch mit 3,3V betrieben werden kann,
     das Display aber dennoch funktioniert.
   ------------------------------------------------------- */

/*
      Default Displayanschluss an den Controller
      ----------------------------------------------------------
         o +5V
         |                            Display                    AVR
         _                        Funktion   PIN             PIN   Funktion
        | |                                                 (ATmega 28 pol.)
        |_|                          GND      1 ------------
         |                          +5V       2 ------------
         o----o Kontrast   ---    Kontrast    3 ------------
        _|_                           RS      4 ------------  19     PB5
        \ /                        rd /wr     5 ------------  GND
        -'-                    (Takt) E       6 ------------  18     PB4
         |                           D0       7                      n.c.
        ---                          D1       8                      n.c.
                                     D2       9                      n.c.
                                     D3      10                      n.c.
                                     D4      11 ------------  26     PC3
                                     D5      12 ------------  17     PB3
                                     D6      13 ------------  27     PC4
                                     D7      14 ------------  16     PB2

*/

#ifndef in_txlcd
  #define in_txlcd

  #include <util/delay.h>
  #include <avr/io.h>
  #include <avr/pgmspace.h>
  #include <stdint.h>

  #include "avr_gpio.h"

  /* -------------------------------------------------------
        Pinbelegung
     ------------------------------------------------------- */

  // command / data
  #define LCD_RS_PORT    PB
  #define LCD_RS_BIT     5

  // clock
  #define LCD_E_PORT     PB
  #define LCD_E_BIT      4

  #define LCD_D4_PORT    PC
  #define LCD_D4_BIT     3

  #define LCD_D5_PORT    PB
  #define LCD_D5_BIT     3

  #define LCD_D6_PORT    PC
  #define LCD_D6_BIT     4

  #define LCD_D7_PORT    PB
  #define LCD_D7_BIT     2


  #define txlcd_prints(tx)      (txlcd_putromstring(PSTR(tx)))       // Anzeige String aus Flashrom: prints("Text");

  /* -------------------------------------------------------
       diverse Macros
     ------------------------------------------------------- */

  #define testbit(reg,pos) ((reg) & (1<<pos))               // testet an der Bitposition pos das Bit auf 1 oder 0


  /* -------------------------------------------------------
       Prototypen
     ------------------------------------------------------- */

    void txlcd_init(void);
    void txlcd_setuserchar(uint8_t nr, const uint8_t *userchar);
    void gotoxy(uint8_t x, uint8_t y);
    void txlcd_putchar(char ch);
    void txlcd_putramstring(uint8_t *c);
    void txlcd_putromstring(const uint8_t *dataPtr);

    #define clrscr()      txlcd_init()

    extern uint8_t wherex,wherey;


  // ----------------------------------------------------------------
  // Praeprozessormacros um 2 Stringtexte zur weiteren Verwendung
  // innerhalb des Praeprozessors  zu verknuepfen
  //
  // Bsp.:
  //        #define ionr      A
  //        #define ioport    conc2(PORT, ionr)
  //
  //        ioport wird nun als "PORTA" behandelt
  #define CONC2EXP(a,b)     a ## b
  #define conc2(a,b)        CONC2EXP(a, b)
  // ----------------------------------------------------------------

  // ----------------------------------------------------------------
  //   Makros zum Initialiseren der verwendeten Pins als Ausgaenge
  //   setzt aus oben angegebenen Portbits neue Makros zusammen.
  //   Bsp.:
  //             #define LCD_D4_PORT  PB
  //             #define LCD_D4_BIT   1
  //   generiert:
  //             #define d4_set()     PB1_input_init()
  //             #define d4_clr()     PB1_clr()
  //             #define d4_output()  PB1_output_init()
  // ----------------------------------------------------------------

  // ---------------- LCD_D4 ---------------
  #define d4_tmp            conc2(LCD_D4_BIT,_set())
  #define d4_set()          conc2(LCD_D4_PORT,d4_tmp)

  #define d4_tmp2           conc2(LCD_D4_BIT,_output_init())
  #define d4_output()       conc2(LCD_D4_PORT,d4_tmp2)

  #define d4_tmp3           conc2(LCD_D4_BIT,_clr())
  #define d4_clr()          conc2(LCD_D4_PORT,d4_tmp3)

  // ---------------- LCD_D5 ---------------
  #define d5_tmp            conc2(LCD_D5_BIT,_set())
  #define d5_set()          conc2(LCD_D5_PORT,d5_tmp)

  #define d5_tmp2           conc2(LCD_D5_BIT,_output_init())
  #define d5_output()       conc2(LCD_D5_PORT,d5_tmp2)

  #define d5_tmp3           conc2(LCD_D5_BIT,_clr())
  #define d5_clr()          conc2(LCD_D5_PORT,d5_tmp3)

  // ---------------- LCD_D6 ---------------
  #define d6_tmp            conc2(LCD_D6_BIT,_set())
  #define d6_set()          conc2(LCD_D6_PORT,d6_tmp)

  #define d6_tmp2           conc2(LCD_D6_BIT,_output_init())
  #define d6_output()       conc2(LCD_D6_PORT,d6_tmp2)

  #define d6_tmp3           conc2(LCD_D6_BIT,_clr())
  #define d6_clr()          conc2(LCD_D6_PORT,d6_tmp3)

  // ---------------- LCD_D7 ---------------
  #define d7_tmp            conc2(LCD_D7_BIT,_set())
  #define d7_set()          conc2(LCD_D7_PORT,d7_tmp)

  #define d7_tmp2           conc2(LCD_D7_BIT,_output_init())
  #define d7_output()       conc2(LCD_D7_PORT,d7_tmp2)

  #define d7_tmp3           conc2(LCD_D7_BIT,_clr())
  #define d7_clr()          conc2(LCD_D7_PORT,d7_tmp3)

  // --------------- LCD_E (clk) -----------
  #define e_tmp             conc2(LCD_E_BIT,_set())
  #define e_set()           conc2(LCD_E_PORT,e_tmp)

  #define e_tmp2            conc2(LCD_E_BIT,_output_init())
  #define e_output()        conc2(LCD_E_PORT,e_tmp2)

  #define e_tmp3            conc2(LCD_E_BIT,_clr())
  #define e_clr()           conc2(LCD_E_PORT,e_tmp3)

  // -----------------LCD_RS ---------------

  #define rs_tmp             conc2(LCD_RS_BIT,_set())
  #define rs_set()           conc2(LCD_RS_PORT,rs_tmp)

  #define rs_tmp2            conc2(LCD_RS_BIT,_output_init())
  #define rs_output()        conc2(LCD_RS_PORT,rs_tmp2)

  #define rs_tmp3            conc2(LCD_RS_BIT,_clr())
  #define rs_clr()           conc2(LCD_RS_PORT,rs_tmp3)

#endif
