/* ---------------------------------------------------------------
                                shiftout.c

     Header fuer Softwaremodul universelles serielle
     Schiebeausgabe mittels Bitbanging

     In der globalen struct Variable
                       "shiftparam"
     werden die Schiebeparameter festgelegt:

            shiftparam.msbfirst   : 1 = MSB zuerst (default)
                                    0 = LSB zuerst
            shiftparam.clkol      : 1 = Taktleitung ist im (default)
                                        Ruhezustand 0
                                    0 = Ruhezustand 1
            shiftparam.clkcycles  : Anzahl Wiederholungen von
                                    _delay_us(1) waehrend des
                                    Taktimpulses
                                    Defaultwert = 1

     Schiebetaktfrequenz bei F_CPU= 16 MHz


     shiftparam.clkcycles |   0     |     1   |    10    |  100  |
     -------------------------------------------------------------
     ungefaehre Taktfreq. | 166 kHz |  85 kHz | 17.5 kHz | 2 kHz |



     MCU     : alle AVR
     F_CPU   :

     04.12.2019   R.Seelig
   --------------------------------------------------------------- */

#ifndef in_shift_out
  #define in_shift_out

  #include <util/delay.h>
  #include <avr/io.h>


  // Anschlusspins fuer die Schiebeausgabe
  #define shift_port          B
  #define shift_portnum       1

  #define shift_clkport       B
  #define shift_clkportnum    2

  // Default Parameterwerte
  #define clkpol_default      1
  #define msbfirst_default    1
  #define clkcycles_default   1


  // Struktur der Optionsparameter fuer das Schieben
  typedef struct shiftsettings
  {
    uint16_t clkcycles;
    uint8_t  clkpol;
    uint8_t  msbfirst;
  } shiftsettings;

  // globale Variable in shiftout.c, nimmt Schiebeparameter auf
  extern struct shiftsettings shiftparam;

  // Prototypen
  void shiftout_init(void);
  void shiftout(uint8_t value);


  // ----------------------------------------------------------------
  //   ab hier nur "Magic" Praeprozessormakros zum Einrichten der
  //   Anschlusspins
  // ----------------------------------------------------------------

  // ----------------------------------------------------------------
  // Praeprozessormacros um 2 Stringtexte zur weiteren Verwendung
  // innerhalb des Praeprozessors  zu verknuepfen
  //
  // Bsp.:
  //        #define ionr      A
  //        #define ioport    CONCAT(PORT, ionr)
  //
  //        ioport wird nun als "PORTA" behandelt

  #define CONC2EXP(a,b)     a ## b
  #define CONCAT(a,b)       CONC2EXP(a, b)
  // ----------------------------------------------------------------

  /* ----------------------------------------------------------------
       Makros zum Initialisieren sowie setzen und reseten der an
       der seriellen Ausgabe beteiligten Anschlusspins
    ---------------------------------------------------------------- */

  // fuer den Anschluss des Datenbits
  #define SHIFTPORT           CONCAT(PORT, shift_port)
  #define SHIFTDDR            CONCAT(DDR,  shift_port)

  #define shift_mask          (1 << shift_portnum)

  #define shift_bit_init()    SHIFTDDR |= shift_mask
  #define shift_bit_set()     SHIFTPORT |= shift_mask
  #define shift_bit_clr()     SHIFTPORT &= ~(shift_mask)

  // fuer den Anschluss des Taktpins
  #define SHIFTCLKPORT        CONCAT(PORT, shift_clkport)
  #define SHIFTCLKDDR         CONCAT(DDR,  shift_clkport)

  #define shiftclk_mask       (1 << shift_clkportnum)

  #define shift_clk_init()    SHIFTCLKDDR |= shiftclk_mask
  #define shift_clk_set()     SHIFTCLKPORT |= shiftclk_mask
  #define shift_clk_clr()     SHIFTCLKPORT &= ~(shiftclk_mask)


#endif
