/* ----------------------------------------------------------
                          shiftreg.h

     Header Softwaremodul zur Ansteuerung der beliebten
     seriellen Schieberegister HEF4094 oder SN74HC595
     (beide Schieberegister funktionieren aehnlich), haben
     jedoch unterschiedliche Pinbelegungen)

     Primaere Funktion der Schieberegister in Verbindung mit
     Mikrocontroller ist die Erweiterung von Ausgangspins

     Dieses Softwaremodul beinhaltet u.a. Bitmuster zur
     Ansteuerung von 7-Segmentanzeigen (gemeinsame Anode
     UND gemeinsame Kathode).

     Einstellungen in shiftreg.h

     MCU       :  alle AVR (da nur Bitbanging)

     21.01.2020  R. Seelig
   ---------------------------------------------------------- */

/*

              HEF4094                             74HC595
          Anschlusspins IC                    Anschlusspins IC

           +------------+                      +------------+
           |            |                      |            |
       str | 1   H   16 | +Vcc            out1 | 1   H   16 | +Vcc
       dat | 2   E   15 | oe              out2 | 2   E   15 | out0
       clk | 3   F   14 | out4            out3 | 3   F   14 | dat
      out0 | 4       13 | out5            out4 | 4       13 | /oe
      out1 | 5   4   12 | out6            out5 | 5   4   12 | strobe
      out2 | 6   0   11 | out7            out6 | 6   0   11 | clk
      out3 | 7   9   10 | out_seriell     out7 | 7   9   10 | /rst
       gnd | 8   4    9 | out_seriell*     gnd | 8   4    9 | out7s
           |            |                      |            |
           +------------+                      +------------+
      * ungepufferter Ausgang

               Anschlussbelegung

      AVR         Pin - HEF4094           Pin - SN74HC595
   ----------------------------------------------------------------
                   16 Vcc                  16 Vcc
                   15 oe = Vcc             10 (/maser reset) = Vcc
      PA2    ...    2 (seriell data)  ...  14 (seriell data)
      PA3    ...    1 (strobe)        ...  12 (strobe)
      PA1    ...    3 (clock)         ...  11 (clock)
                   10 (seriell out')        9 (seriell out; Q7S)
                   8 GND                    8 GND

/*
  Beispiel Anbindung 7-Segmentanzeige an Schieberegister

       SA52
 (gemeinsame Anode)

         G            Zuordnung Segmente Anzeige zu SR-Ausgaengen
     g f A a b
    +#-#-#-# #+       SR-Out  |  7 | 6 | 5 | 4 | 3 | 2 | 1 | 0 |
    |   _a_   |       ------------------------------------------
    | f|_g_|b |       Segment | dp | g | f | e | d | c | b | a |
    | e|___|c |       ------------------------------------------
    |    d    |
    +#-#-#-#-#+
     e d G c d
         A   p
*/

#ifndef in_shiftreg
  #define in_shiftreg

  #include <avr/io.h>
  #include <avr/pgmspace.h>
  #include <stdint.h>

  // Zuordnung der Anschluesse des Schieberegisters an den AVR

  // Dataanschluss
  #define sr_datport     A
  #define sr_datbitnr    2

  // Clockanschluss
  #define sr_clkport     A
  #define sr_clkbitnr    1

  // Strobeanschluss
  #define sr_strport     A
  #define sr_strbitnr    3


  // Bitmuster fuer 7-Segmentanzeigen. Nur eine von beiden steht zur Wahl (oder keine)

  #define  gAnode         1               // Bitmuster fuer gemeinsame Anode
  #define  gKathode       0               // Bitmuster fuer gemeinsame Kathode

  #if ((gKathode == 1) || (gAnode == 1))
    extern const uint8_t s7bmp[] PROGMEM;
  #endif


  // globale Puffervariable des Schieberegisters. Beinhaltet den aktuellen
  // Ausgabezustands des Schiebregisters.
  extern uint8_t sr_value;

  /* ----------------------------------------------------------
                             Prototypen
     ---------------------------------------------------------- */

  void sr_setvalue(uint8_t value);
  void sr_setkaskade(uint8_t anz, uint8_t *datas);
  void sr_setsinglebit(char nr);
  void sr_clrsinglebit(char nr);

  /* ----------------------------------------------------------
                      "User useable" Macros
     ---------------------------------------------------------- */
  #define sr_init()         { srdat_init(); srclk_init(); srstrobe_init(); }

  #define sr_set7seg(nr)    sr_setvalue(pgm_read_byte(&(s7bmp[nr])))

  #define sr_clear()     sr_setvalue(0x00)


  // ----------------------------------------------------------------
  // Praeprozessormacros um 2 Stringtexte zur weiteren Verwendung
  // innerhalb des Praeprozessors  zu verknuepfen
  //
  // Bsp.:
  //        #define ionr      A
  //        #define ioport    conc2(PORT, ionr)
  //
  //        ioport wird nun als "PORTA" behandelt
  #define conc2(a,b)        CONC2EXP(a, b)
  #define CONC2EXP(a,b)     a ## b
  // ----------------------------------------------------------------

  // ----------------------------------------------------------------
  //   Makros zum Initialiseren der verwendeten Pins als Ausgaenge
  //   sowie zum Setzen / Loeschen dieser Pins (Bitbanging)
  // ----------------------------------------------------------------
  #define datport           conc2(PORT,sr_datport)
  #define datddr            conc2(DDR,sr_datport)
  #define clkport           conc2(PORT,sr_clkport)
  #define clkddr            conc2(DDR,sr_clkport)
  #define strport           conc2(PORT,sr_strport)
  #define strddr            conc2(DDR,sr_strport)

  #define srdat_init()      datddr|= (1 << sr_datbitnr)
  #define srdat_set()       ( datport|= (1 << sr_datbitnr) )
  #define srdat_clr()       ( datport&= (~(1 << sr_datbitnr)) )


  #define srclk_init()      clkddr|= (1 << sr_clkbitnr)
  #define srclk_set()       ( clkport|= (1 << sr_clkbitnr) )
  #define srclk_clr()       ( clkport&= (~(1 << sr_clkbitnr)) )

  #define srstrobe_init()   strddr|= (1 << sr_strbitnr)
  #define srstrobe_set()    ( strport|= (1 << sr_strbitnr) )
  #define srstrobe_clr()    ( strport&= (~(1 << sr_strbitnr)) )

#endif
