/* ----------------------------------------------------------
                          shiftreg2.h

     Header Softwaremodul zur Ansteuerung der seriellen
     Schieberegister HEF4094 oder SN74HC595. Beide Schiebe-
     register funktionieren aehnlich, haben jedoch
     unterschiedliche Pinbelegungen.

     Primaere Funktion der Schieberegister in Verbindung mit
     Mikrocontroller ist die Erweiterung von Ausgangspins
     (GPIO - Porterweiterung)

     MCU       :  alle AVR (da nur Bitbanging)

     21.01.2020  R. Seelig
   ---------------------------------------------------------- */

/*

              HEF4094                             74HC595
          Anschlusspins IC                    Anschlusspins IC

           +------------+                      +------------+
           |            |                      |            |
       str | 1   H   16 | +Vcc            out1 | 1   H   16 | +Vcc
       dat | 2   E   15 | oe              out2 | 2   E   15 | out0
       clk | 3   F   14 | out4            out3 | 3   F   14 | dat
      out0 | 4       13 | out5            out4 | 4       13 | /oe
      out1 | 5   4   12 | out6            out5 | 5   4   12 | strobe
      out2 | 6   0   11 | out7            out6 | 6   0   11 | clk
      out3 | 7   9   10 | out_seriell     out7 | 7   9   10 | /rst
       gnd | 8   4    9 | out_seriell*     gnd | 8   4    9 | out7s
           |            |                      |            |
           +------------+                      +------------+
      * ungepufferter Ausgang


                           Anschlussbelegung

   Arduino   AVR      Pin - HEF4094           Pin - SN74HC595
   ----------------------------------------------------------------
                       16 Vcc                  16 Vcc
                       15 oe = Vcc             10 (/maser reset) = Vcc
     D11     PB3 ...    2 (seriell data)  ...  14 (seriell data)
     D7      PD7 ...    1 (strobe)        ...  12 (strobe)
     D13     PB5 ...    3 (clock)         ...  11 (clock)
                       10 (seriell out')        9 (seriell out; Q7S)
                        8 GND                   8 GND
*/

#ifndef in_shiftreg2
  #define in_shiftreg2

  #include <avr/io.h>
  #include <avr/pgmspace.h>
  #include <stdint.h>

  // Zuordnung der Anschluesse des Schieberegisters an den AVR

  // Dataanschluss
  #define sr_datport     B
  #define sr_datbitnr    3

  // Clockanschluss
  #define sr_clkport     B
  #define sr_clkbitnr    5

  // Strobeanschluss
  #define sr_strport     D
  #define sr_strbitnr    7


  // globale Puffervariable des Schieberegisters. Beinhaltet den aktuellen
  // Ausgabezustands des Schiebregisters.
  extern uint8_t sr_value;

  /* ----------------------------------------------------------
                             Prototypen
     ---------------------------------------------------------- */

  void sr_setvalue(uint8_t value);
  void sr_setkaskade(uint8_t anz, uint8_t *datas);
  void sr_setarraykaskade(uint8_t anz, uint8_t dir, uint8_t *datas);
  void sr_setsinglebit(char nr);
  void sr_clrsinglebit(char nr);

  /* ----------------------------------------------------------
                      "User useable" Macros
     ---------------------------------------------------------- */
  #define sr_init()         { srdat_init(); srclk_init(); srstrobe_init();      \
                              srclk_clr(); srdat_clr(); srstrobe_clr(); }

  #define sr_set7seg(nr)    sr_setvalue(pgm_read_byte(&(s7bmp[nr])))

  #define sr_clear()        sr_setvalue(0x00)


  // ----------------------------------------------------------------
  // Praeprozessormacros um 2 Stringtexte zur weiteren Verwendung
  // innerhalb des Praeprozessors  zu verknuepfen
  //
  // Bsp.:
  //        #define ionr      A
  //        #define ioport    conc2(PORT, ionr)
  //
  //        ioport wird nun als "PORTA" behandelt
  #define conc2(a,b)        CONC2EXP(a, b)
  #define CONC2EXP(a,b)     a ## b
  // ----------------------------------------------------------------

  // ----------------------------------------------------------------
  //   Makros zum Initialiseren der verwendeten Pins als Ausgaenge
  //   sowie zum Setzen / Loeschen dieser Pins (Bitbanging)
  // ----------------------------------------------------------------
  #define datport           conc2(PORT,sr_datport)
  #define datddr            conc2(DDR,sr_datport)
  #define clkport           conc2(PORT,sr_clkport)
  #define clkddr            conc2(DDR,sr_clkport)
  #define strport           conc2(PORT,sr_strport)
  #define strddr            conc2(DDR,sr_strport)

  #define srdat_init()      datddr|= (1 << sr_datbitnr)
  #define srdat_set()       ( datport|= (1 << sr_datbitnr) )
  #define srdat_clr()       ( datport&= (~(1 << sr_datbitnr)) )


  #define srclk_init()      clkddr|= (1 << sr_clkbitnr)
  #define srclk_set()       ( clkport|= (1 << sr_clkbitnr) )
  #define srclk_clr()       ( clkport&= (~(1 << sr_clkbitnr)) )

  #define srstrobe_init()   strddr|= (1 << sr_strbitnr)
  #define srstrobe_set()    ( strport|= (1 << sr_strbitnr) )
  #define srstrobe_clr()    ( strport&= (~(1 << sr_strbitnr)) )

#endif
