/* ------------------------------------------------------------

                        tx4_n5110.h

   Headerdatei fuer die Anbindung eines s/w Nokia LCD an
   einen ATtiny. Abgespeckte LCD-Bibliothek, keine Grafik-
   funktionen

   17.08.2017 R. Seelig
   ---------------------------------------------------------- */

/*

                                                 ATtiny44
                                             Anschlusspins IC

                                              +-----------+
                                         Vcc  | 1      14 |  GND
                   PCINT8 - XTAL1 - CLKI PB0  | 2   A  13 |  PA0 - ADC0 - AREF - PCINT0
                        PCINT9 - XTAL2 - PB1  | 3   T  12 |  PA1 - ADC1 - AIN0 - PCINT1
                 PCINT11 - /reset - dW - PB3  | 4   t  11 |  PA2 - ADC2 - AIN1 - PCINT2
         PCINT10 - INT0 - OC0A - CKOUT - PB2  | 5   i  10 |  PA3 - ADC3 - T0 - PCINT3
            PCINT7 - ICP - OC0B - ADC7 - PA7  | 6   n   9 |  PA4 - ADC4 - USCK - SCL - T1 - PCINT4
PCINT6 - OC1A - SDA - MOSI - DI - ADC6 - PA6  | 7   y   8 |  PA5 - ADC5 - DO - MISO - OC1B PCINT5
                                              +-----------+
Hinweis:

    bei Verwendung von USI als Hardware SPI ist der sendende Ausgang
    D0. Das heisst: PA5 entspricht hier dann MOSI (im Gegensatz zur
    Verwendung von ISP bei der der Anschluss PA5 MISO ist) !!!!


    Wiring:

       ATTiny44             Display
    ---------------------------------

       PA4  (9)   ---       CLK (5)
       PA5  (8)   ---       DIN (4)
       PA3 (10)   ---       DC  (3)
       PA2 (11)   ---       CE  (2)
       PA1 (12)   ---       RST (1)


*/

#ifndef in_tx4_n5110
  #define in_tx4_n5110

  #include <util/delay.h>
  #include <util/atomic.h>
  #include <avr/io.h>
  #include <avr/pgmspace.h>


  #define hardware_spi                 1        // 0 : SPI ueber Bitbanging realisieren, hier
                                                //     duerfen dann die Anschlusspins veraendert werden
                                                // 1 : SPI ueber USI-Hardware realisieren,
                                                //     Anschlusspins fuer Mosi, Miso, Clk duerfen
                                                //     NICHT veraendert werden

  #define fullascii                             // ASCII Zeichensatz wird auch mit
                                                // Kleinbuchstaben implementiert

  #ifdef fullascii
    #define lastascii 126                       // letztes angegebenes Asciizeichen
  #else
    #define lastascii 96
  #endif

  #define SPI_DDR                 DDRA
  #define SPI_PORT                PORTA
  #define SPI_MOSI_PIN            PA5
  #define SPI_MISO_PIN            PA6
  #define SPI_CLK_PIN             PA4
  #define SPI_SS_PIN              PA2

  #define LCD_PORT                PORTA
  #define LCD_DDR                 DDRA
  #define LCD_RST_PIN             PA1
  #define LCD_DC_PIN              PA3
  #define LCD_CE_PIN              SPI_SS_PIN


  #ifdef n3410
    // Nokia 3410

    #define LCD_VISIBLE_X_RES     96
    #define LCD_VISIBLE_Y_RES     64
    #define LCD_REAL_X_RES        102
    #define LCD_REAL_Y_RES        72
    #define OK                    0
    #define OUT_OF_BORDER         1

  #else
    // Nokia 5110 / 3310

    #define LCD_VISIBLE_X_RES     84
    #define LCD_VISIBLE_Y_RES     48
    #define LCD_REAL_X_RES        84
    #define LCD_REAL_Y_RES        48
    #define OK                    0
    #define OUT_OF_BORDER         1
  #endif

  /* ----------------------------------------------------
                          Prototypen
     ---------------------------------------------------- */

  void spi_init(void);
  unsigned char spi_out(uint8_t data);
  void wrcmd(uint8_t command);                            // sende ein Kommando
  void lcd_init();                                        // initialisiere das Display
  void wrdata(uint8_t dat);                               // sende ein Datum
  void clrscr();                                          // loesche das Display
  void gotoxy(char x,char y);                             // positioniere Ausgabeposition
  void lcd_putchar_d(char ch);                            // setze Zeichen auf das Display
  void putramstring(char *c);                             // schreibe String aus dem RAM
  void putromstring(const unsigned char *dataPtr);        // dto. ROM

  #define prints(tx)     (putromstring(PSTR(tx)))         // Anzeige String aus Flashrom: prints("Text");
  #define printa(tx)     (putramstring(tx))               // Anzeige eines Strings der in einem Array im RAM liegt


  /* ----------------------------------------------------
     Variable der Displayroutinen
     ---------------------------------------------------- */

  extern char wherex;
  extern char wherey;
  extern char invchar;               // = 1 fuer inversive Textausgabe


  // --------------------------------------------------------------
  // Zeichensatz
  // --------------------------------------------------------------


  /* Bitmaps des Ascii-Zeichensatzes
     ein Smily wuerde so aussehen:
        { 0x36, 0x46, 0x40, 0x46, 0x36 }  // Smiley

     ein grosses A ist folgendermassen definiert:

     { 0x7E, 0x11, 0x11, 0x11, 0x7E }

     . x x x x x x .
     . . . x . . . x
     . . . x . . . x
     . . . x . . . x
     . x x x x x x .

  */

  static const unsigned char fonttab [][5] PROGMEM = {
      { 0x00, 0x00, 0x00, 0x00, 0x00 },   // space
      { 0x00, 0x00, 0x2f, 0x00, 0x00 },   // !
      { 0x00, 0x07, 0x00, 0x07, 0x00 },   // "
      { 0x14, 0x7f, 0x14, 0x7f, 0x14 },   // #
      { 0x24, 0x2a, 0x7f, 0x2a, 0x12 },   // $
      { 0xc4, 0xc8, 0x10, 0x26, 0x46 },   // %
      { 0x36, 0x49, 0x55, 0x22, 0x50 },   // &
      { 0x00, 0x05, 0x03, 0x00, 0x00 },   // '
      { 0x00, 0x1c, 0x22, 0x41, 0x00 },   // (
      { 0x00, 0x41, 0x22, 0x1c, 0x00 },   // )
      { 0x14, 0x08, 0x3E, 0x08, 0x14 },   // *
      { 0x08, 0x08, 0x3E, 0x08, 0x08 },   // +
      { 0x00, 0x00, 0x50, 0x30, 0x00 },   // ,
      { 0x10, 0x10, 0x10, 0x10, 0x10 },   // -
      { 0x00, 0x60, 0x60, 0x00, 0x00 },   // .
      { 0x20, 0x10, 0x08, 0x04, 0x02 },   // /
      { 0x3E, 0x51, 0x49, 0x45, 0x3E },   // 0
      { 0x00, 0x42, 0x7F, 0x40, 0x00 },   // 1
      { 0x42, 0x61, 0x51, 0x49, 0x46 },   // 2
      { 0x21, 0x41, 0x45, 0x4B, 0x31 },   // 3
      { 0x18, 0x14, 0x12, 0x7F, 0x10 },   // 4
      { 0x27, 0x45, 0x45, 0x45, 0x39 },   // 5
      { 0x3C, 0x4A, 0x49, 0x49, 0x30 },   // 6
      { 0x01, 0x71, 0x09, 0x05, 0x03 },   // 7
      { 0x36, 0x49, 0x49, 0x49, 0x36 },   // 8
      { 0x06, 0x49, 0x49, 0x29, 0x1E },   // 9
      { 0x00, 0x36, 0x36, 0x00, 0x00 },   // :
      { 0x00, 0x56, 0x36, 0x00, 0x00 },   // ;
      { 0x08, 0x14, 0x22, 0x41, 0x00 },   // <
      { 0x14, 0x14, 0x14, 0x14, 0x14 },   // =
      { 0x00, 0x41, 0x22, 0x14, 0x08 },   // >
      { 0x02, 0x01, 0x51, 0x09, 0x06 },   // ?
      { 0x32, 0x49, 0x59, 0x51, 0x3E },   // @
      { 0x7E, 0x11, 0x11, 0x11, 0x7E },   // A
      { 0x7F, 0x49, 0x49, 0x49, 0x36 },   // B
      { 0x3E, 0x41, 0x41, 0x41, 0x22 },   // C
      { 0x7F, 0x41, 0x41, 0x22, 0x1C },   // D
      { 0x7F, 0x49, 0x49, 0x49, 0x41 },   // E
      { 0x7F, 0x09, 0x09, 0x09, 0x01 },   // F
      { 0x3E, 0x41, 0x49, 0x49, 0x7A },   // G
      { 0x7F, 0x08, 0x08, 0x08, 0x7F },   // H
      { 0x00, 0x41, 0x7F, 0x41, 0x00 },   // I
      { 0x20, 0x40, 0x41, 0x3F, 0x01 },   // J
      { 0x7F, 0x08, 0x14, 0x22, 0x41 },   // K
      { 0x7F, 0x40, 0x40, 0x40, 0x40 },   // L
      { 0x7F, 0x02, 0x0C, 0x02, 0x7F },   // M
      { 0x7F, 0x04, 0x08, 0x10, 0x7F },   // N
      { 0x3E, 0x41, 0x41, 0x41, 0x3E },   // O
      { 0x7F, 0x09, 0x09, 0x09, 0x06 },   // P
      { 0x3E, 0x41, 0x51, 0x21, 0x5E },   // Q
      { 0x7F, 0x09, 0x19, 0x29, 0x46 },   // R
      { 0x46, 0x49, 0x49, 0x49, 0x31 },   // S
      { 0x01, 0x01, 0x7F, 0x01, 0x01 },   // T
      { 0x3F, 0x40, 0x40, 0x40, 0x3F },   // U
      { 0x1F, 0x20, 0x40, 0x20, 0x1F },   // V
      { 0x3F, 0x40, 0x38, 0x40, 0x3F },   // W
      { 0x63, 0x14, 0x08, 0x14, 0x63 },   // X
      { 0x07, 0x08, 0x70, 0x08, 0x07 },   // Y
      { 0x61, 0x51, 0x49, 0x45, 0x43 },   // Z
      { 0x00, 0x7F, 0x41, 0x41, 0x00 },   // [
      { 0x55, 0x2A, 0x55, 0x2A, 0x55 },   // "Yen"
      { 0x00, 0x41, 0x41, 0x7F, 0x00 },   // ]
      { 0x04, 0x02, 0x01, 0x02, 0x04 },   // ^
      { 0x40, 0x40, 0x40, 0x40, 0x40 },   // _
      { 0x00, 0x01, 0x02, 0x04, 0x00 },   // '

  #ifdef fullascii
      { 0x20, 0x54, 0x54, 0x54, 0x78 },   // a
      { 0x7F, 0x48, 0x44, 0x44, 0x38 },   // b
      { 0x38, 0x44, 0x44, 0x44, 0x20 },   // c
      { 0x38, 0x44, 0x44, 0x48, 0x7F },   // d
      { 0x38, 0x54, 0x54, 0x54, 0x18 },   // e
      { 0x08, 0x7E, 0x09, 0x01, 0x02 },   // f
      { 0x0C, 0x52, 0x52, 0x52, 0x3E },   // g
      { 0x7F, 0x08, 0x04, 0x04, 0x78 },   // h
      { 0x00, 0x44, 0x7D, 0x40, 0x00 },   // i
      { 0x20, 0x40, 0x44, 0x3D, 0x00 },   // j
      { 0x7F, 0x10, 0x28, 0x44, 0x00 },   // k
      { 0x00, 0x41, 0x7F, 0x40, 0x00 },   // l
      { 0x7C, 0x04, 0x18, 0x04, 0x78 },   // m
      { 0x7C, 0x08, 0x04, 0x04, 0x78 },   // n
      { 0x38, 0x44, 0x44, 0x44, 0x38 },   // o
      { 0x7C, 0x14, 0x14, 0x14, 0x08 },   // p
      { 0x08, 0x14, 0x14, 0x18, 0x7C },   // q
      { 0x7C, 0x08, 0x04, 0x04, 0x08 },   // r
      { 0x48, 0x54, 0x54, 0x54, 0x20 },   // s
      { 0x04, 0x3F, 0x44, 0x40, 0x20 },   // t
      { 0x3C, 0x40, 0x40, 0x20, 0x7C },   // u
      { 0x1C, 0x20, 0x40, 0x20, 0x1C },   // v
      { 0x3C, 0x40, 0x30, 0x40, 0x3C },   // w
      { 0x44, 0x28, 0x10, 0x28, 0x44 },   // x
      { 0x0C, 0x50, 0x50, 0x50, 0x3C },   // y
      { 0x44, 0x64, 0x54, 0x4C, 0x44 },   // z
      // Zeichen vom Ascii-Satz abweichend
      { 0x3E, 0x7F, 0x7F, 0x3E, 0x00 },   // Zeichen 123 : ausgefuelltes Oval
      { 0x06, 0x09, 0x09, 0x06, 0x00 },   // Zeichen 124 : hochgestelltes kleines o (fuer Gradzeichen);
      { 0x01, 0x01, 0x01, 0x01, 0x01 },   // Zeichen 125 : Strich in der obersten Reihe
      { 0x00, 0x1D, 0x15, 0x17, 0x00 }    // Zeichen 126 : "hoch 2"
  #endif
  };

#endif

