/* -----------------------------------------------
                    kty81-210.c

     Auswertung eines KTY81-210 Temperatursensors

     MCU : ATmega8, ATmega48/88/168/328p
     Takt:

     27.01.2020  R. Seelig
   ----------------------------------------------- */

#include <avr/io.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <stdio.h>
#include <stdint.h>

#include "uart_all.h"
#include "my_printf.h"


#define printf    my_printf
#define delay     _delay_ms

#define widoffset      -8;                   // hier kann ein "Abgleich" des Sensors erfolgen
                                             // Korrekturwert beim Versuchsaufbau des Authors

#define tabanz          11
#define kty81_firstval  -20.0;

// Widerstandstabelle des KTY81-210 in Ohm, 10 Grad Celisusschritte, beginnend bei
// -20 Grad   (Spannungsteiler 2,2k-Ohm mit KTY81)
uint16_t kty81_tab [tabanz] =
  { 1367, 1495, 1630, 1772, 1922, 2080, 2245, 2417, 2597, 2785, 2980 };


/* -----------------------------------------------
                    adc_init

     initialisieren eines ADC-Kanals

   Uebergabe:

     refmode bezeichnet die zu verwendende
     Spannungsreferenz
       0 : externe Referenz an Aref
       1 : +AVcc mit ext. Kondensator
       2 : reserviert
       3 : interne Referenz: 1,1V ATmega168 / 2,56V ATmega8

     channel bezeichnet den Kanal auf dem gemessen werden soll

     Spannungsreferenz ATMEGAxx8:

  Bit: 7      6
     REFS1  REFS0      Referenz
       0      0        ext. Referenz an  Aref
       0      1        +AVcc mit ext. Kondensator
       1      0        reserviert
       1      1        Interne Spannungsreferenz: 1,1V (ATmegaxx8) / 2,56V (ATmega8)

    Bit5: ADLAR = 0 (rechtsbuendig); MUX= 0 => ADC0 gewaehlt

  Bit3..0: Selektor Analogmultiplexer

     // ADMUX= 1<<REFS0;                                          // AVcc (Betriebsspannung) als Referenz
     // ADMUX= 1<<REFS0 | 1<<REFS1;                               // int. Referenz
   ----------------------------------------------- */
void adc_init(uint8_t refmode, uint8_t channel)
{

   ADMUX = refmode << 6;
   ADMUX |= channel;

   ADCSRA= 0xe7;                             // ADC enable, single gestartet, free running mode
                                            // Taktteiler / 128
   delay(1);
   ADCSRA= 0xa7;
}

/* -----------------------------------------------
                    getadc_10bit

     liest einen analogen Wert auf dem initalisierten
     Kanal ein
   ----------------------------------------------- */
uint16_t getadc_10bit (void)
{
  uint16_t adcwert,adchw;

  adcwert= ADCL;
  adchw= ADCH;
  adchw = adchw << 8;
  adcwert|= adchw;

  return(adcwert);
}


/* --------------------------------------------------------
   my_putchar

   wird von my-printf / printf aufgerufen und hier muss
   eine Zeichenausgabefunktion angegeben sein, auf das
   printf dann schreibt !
   -------------------------------------------------------- */
void my_putchar(char ch)
{
  uart_putchar(ch);
}


/* -----------------------------------------------
                         main
   ----------------------------------------------- */
int main(void)
{
  char     b, cnt;
  uint16_t mwert;
  uint16_t addmess;
  float    rmess;
  float    t,t2, tadd;
  float    tempvalue;

  uart_init();

  printf("\n\rTemperaturerfassung mit KTY81-210");
  printf("\n\r-------------------------------------------\n\n\r");

  _delay_us(30);
  adc_init(1,0);                             // Betriebsspg. als Referenzspannung (Mode 1), Input Channel 0
  addmess= getadc_10bit();

  while(1)
  {
    tempvalue= 0;

    // 10 Messungen durchfuehren
    for (cnt= 0; cnt< 10; cnt++)
    {
      mwert= getadc_10bit();

      rmess= mwert;
      rmess= 2.2 * (rmess / (1023-rmess));   // Spannungsteiler mit 2.2 k-Ohm Vorwiderstand
      rmess= rmess*1000;
      mwert= rmess + widoffset;              // mwert enthaelt den Messwert

      mwert= mwert + widoffset;

      b= 0; t= kty81_firstval;

      while ((mwert > kty81_tab[b]) && (b< tabanz))
      {
        b++;
        t= t + 10.0;
      }

      if (mwert != kty81_tab[b])
      {
        t= t-10;
        tadd= kty81_tab[b]-kty81_tab[b-1];     // Diff. zwischen 2 Tabellenwerte = 10 Grad

        t2= mwert- kty81_tab[b-1];
        t2= (t2 / tadd)*10;
        t= t+t2;
      }

      tempvalue= tempvalue + t;                      // ermittelte Temp.-Werte aufaddieren
                                                     // um sie spaeter als Mittelwert auszuwerten
      delay(30);

    }

    // tempvalue enthaelt 10 aufaddierte Temperaturwerte (und ist somit um den Faktor 10 groesser)
    // Durch Darstellung mit einer Nachkommastelle ist die Anzeige dann wieder "richtig"
    printfkomma= 1;

    printf("\r Temp= %k %cC      ",(uint16_t)tempvalue,'o');
  }
}
