/* --------------------------- my_printf32.c  ----------------------------

     Implementierung einer sehr minimalistischen (und im Funktionsumfang
     eingeschraenkten) Version von printf, jedoch (im Gegensatz zu
     my_printf.c) mit Ausgabe fuer 32-Bit Integer.

     Benoetigt "irgendwo" im Gesamtprogram ein

                my_putchar(char c);


     29.03.2024   R. Seelig

     Danke an www.mikrocontroller.net (Dr.Sommer, beric) fuer deren
     Hilfestelleung

   --------------------------------------------------------------------- */

#include "my_printf32.h"

char printfkomma = 4;

char printfoutstream = MYSTREAM;
char *printfptr;

extern void my_putchar(char ch);

/* ------------------------------------------------------------
     charout

     sendet ein Zeichen in Abhaengigkeit von Outstream ent-
     weder an das Ausgabegeraet oder in ein Chararray
   ------------------------------------------------------------ */
void charout(char ch)
{
  if (printfoutstream == MYSTREAM)
  {
    my_putchar(ch);
  }
  else
  {
    *printfptr= ch;
    printfptr++;
  }
}


/* ------------------------------------------------------------
                             putint

     gibt einen Integer dezimal aus. Ist Uebergabe
     "komma" != 0 wird ein "Kommapunkt" mit ausgegeben.
     Groesste darstellare Zahl ist 999.999.999

     Bsp.: 12345 wird als 123.45 ausgegeben.
     (ermoeglicht Pseudofloatausgaben im Bereich)
   ------------------------------------------------------------ */
void putint(int32_t i, char komma)
{
  #define decade   10

  typedef enum boolean { FALSE, TRUE }bool_t;

  uint32_t zz[]  = { 1000000000, 100000000, 10000000, 1000000, 100000, 10000, 1000, 100, 10 };
  bool_t not_first = FALSE;

  uint8_t zi;
  uint32_t z, b;

  komma= decade-komma;

  if (!i)
  {
    charout('0');
  }
  else
  {
    if(i < 0)
    {
      charout('-');
      i = -i;
    }

    for(zi = 0; zi < (decade-1); zi++)
    {
      z = 0;
      b = 0;

      while(z + zz[zi] <= i)
      {
        b++;
        z += zz[zi];
      }

      if(b || not_first)
      {
        charout('0' + b);
        not_first = TRUE;
      }

      if (zi+1 == komma)
      {
        if (!not_first) charout('0');
        charout('.');
        not_first= TRUE;
      }

      i -= z;
    }
    charout('0' + i);
  }
}

/* ------------------------------------------------------------
                       hexnibbleout

     gibt die unteren 4 Bits eines chars als Hexaziffer aus.
     Eine Pruefung ob die oberen vier Bits geloescht sind
     erfolgt NICHT !
  -------------------------------------------------------------  */
void hexnibbleout(uint8_t b)
{
  if (b< 10) b+= '0'; else b+= 55;
  charout(b);
}

/* ------------------------------------------------------------
                            puthex

     gibt einen Integer hexadezimal aus. Ist die auszugebende
     Zahl >= 0xff erfolgt die Ausgabe 2-stellig, ist sie
     groesser erfolgt die Ausgabe 4-stellig.

     Ist out16 gesetzt, erfolgt ausgabe immer 4 stellig
   ------------------------------------------------------------ */
void puthex(uint16_t h, char out16)
{
  uint8_t b;

  if ((h> 0xff) || out16)                    // 16 Bit-Wert
  {
    b= (h >> 12);
    hexnibbleout(b);
    b= (h >> 8) & 0x0f;
    hexnibbleout(b);
  }
  b= h;
  b= (h >> 4) & 0x0f;
  hexnibbleout(b);
  b= h & 0x0f;
  hexnibbleout(b);
}

/* --------------------------------------------------
                  my_putramstring

     gibt einen String aus dem RAM aus
   -------------------------------------------------- */
void my_putramstring(char *p)
{
  while(*p)
  {
    charout( *p );
    p++;
  }
}

#if (printf_enable == 1)
  /* --------------------------------------------------
                         own_printf

                        als Macros:
                         MY_PRINTF
                        TINY_PRINTF

       alternativer Ersatz fuer printf.

       Aufruf:

           own_printf(PSTR("Ergebnis= %d"),zahl);

       oder durch define-Makro (besser):

           tiny_printf("Ergebnis= %d",zahl);
           my_printf("Ergebnis= %d",zahl);

       Platzhalterfunktionen:

          %s     : Ausgabe Textstring
          %d     : dezimale Ausgabe (16-Bit)
          %D     : dezimale Ausgabe (32-Bit)
          %x     : hexadezimale Ausgabe (16-Bit)
                   ist Wert > 0xff erfolgt 4-stellige Ausgabe
                   ist Wert <= 0xff erfolgt 2-stellige Ausgabe
          %X     : hexadezimale Ausgabe (16-Bit)
                   ist Wert > 0xff erfolgt 4-stellige Ausgabe
                   ist Wert <= 0xff erfolgt 2-stellige Ausgabe
          %k     : 32-Bit Integerausgabe als Pseudokommazahl
                   1234523 wird als 123.4523 ausgegeben (bei
                   printfkomma = 4)
          %c     : Ausgabe als Asciizeichen

     --------------------------------------------------*/

  void own_printf(const uint8_t *s,...)
  {
    int32_t   arg1;
    int16_t   arg16;
    uint32_t  xarg1;
    char     *arg2;
    char      ch;
    va_list   ap;
    uint8_t   token;

    va_start(ap,s);
    do
    {
      ch= pgm_read_byte(s);
      if(ch== 0) return;

      if(ch=='%')            // Platzhalterzeichen
      {
        token= pgm_read_byte(++s);
        switch(token)
        {
          case 'D':          // dezimale Ausgabe (32-Bit)
          {
            arg1= va_arg(ap,int32_t);
            putint(arg1,0);
            break;
          }
          case 'd':          // dezimale Ausgabe (16-Bit)
          {
            arg16= va_arg(ap,int16_t);
            putint(arg16,0);
            break;
          }
          case 'x':          // hexadezimale Ausgabe (16-Bit)
          {
            arg16= va_arg(ap,int16_t);
            puthex(arg16, 0);
            break;
          }
          case 'X':          // hexadezimale Ausgabe (32-Bit)
          {
            xarg1= va_arg(ap,uint32_t);
            if (xarg1 <= 0xFFFF)
            {
              puthex(xarg1, 0);
            }
            else
            {
              puthex(xarg1 >> 16,0);
              puthex(xarg1 & 0xffff,1);
            }

            break;
          }
          case 'k':
          {
            xarg1= va_arg(ap,int32_t);
            putint(xarg1,printfkomma);     // Integerausgabe mit Komma: 12896 zeigt 12.896 an
            break;
          }
          case 'c':          // Zeichenausgabe
          {
            arg1= va_arg(ap,int);
            charout(arg1);
            break;
          }
          case '%':
          {
            charout(token);
            break;
          }
          case 's':
          {
            arg2= va_arg(ap,char *);
            my_putramstring(arg2);
            break;
          }
        }
      }
      else
      {
        charout(ch);
      }
      s++;
    }while (ch != '\0');
  }
#endif

#if (sprintf_enable == 1)
  /* ------------------------------------------------------------
       own_sprintf

                        als Macros:
                         my_sprintf
                        tiny_sprintf

       alternativer Ersatz fuer sprintf.

       Aufruf:

           own_sprintf(textbuffer, PSTR("Ergebnis= %d"),zahl);

       oder durch define-Makro (besser):

           tiny_sprintf(textbuffer, "Ergebnis= %d",zahl);
           my_sprintf("textbuffer, Ergebnis= %d",zahl);

       Platzhalterfunktionen siehe my_printf
     ------------------------------------------------------------ */
  void own_sprintf(char *buffer, const uint8_t *s,...)
  {
    int32_t   arg1;
    int16_t   arg16;
    uint32_t  xarg1;
    char     *arg2;
    char      ch;
    va_list   ap;
    uint8_t   token;

    printfoutstream= MYARRAY;
    printfptr= buffer;

    va_start(ap,s);
    do
    {
      ch= pgm_read_byte(s);
      if(ch== 0)
      {
        *printfptr= 0;
        printfoutstream= MYSTREAM;
        return;
      }

      if(ch=='%')            // Platzhalterzeichen
      {
        token= pgm_read_byte(++s);
        switch(token)
        {
          case 'D':          // dezimale Ausgabe (32-Bit)
          {
            arg1= va_arg(ap,int32_t);
            putint(arg1,0);
            break;
          }
          case 'd':          // dezimale Ausgabe (16-Bit)
          {
            arg16= va_arg(ap,int16_t);
            putint(arg16,0);
            break;
          }
          case 'x':          // hexadezimale Ausgabe (16-Bit)
          {
            arg16= va_arg(ap,int16_t);
            puthex(arg16, 0);
            break;
          }
          case 'X':          // hexadezimale Ausgabe (32-Bit)
          {
            xarg1= va_arg(ap,uint32_t);
            if (xarg1 <= 0xFFFF)
            {
              puthex(xarg1, 0);
            }
            else
            {
              puthex(xarg1 >> 16,0);
              puthex(xarg1 & 0xffff,1);
            }

            break;
          }
          case 'k':
          {
            xarg1= va_arg(ap,int32_t);
            putint(xarg1,printfkomma);     // Integerausgabe mit Komma: 12896 zeigt 12.896 an
            break;
          }
          case 'c':          // Zeichenausgabe
          {
            arg1= va_arg(ap,int);
            charout(arg1);
            break;
          }
          case '%':
          {
            charout(token);
            break;
          }
          case 's':
          {
            arg2= va_arg(ap,char *);
            my_putramstring(arg2);
            break;
          }
        }
      }
      else
      {
        charout(ch);
      }
      s++;
    }while (ch != '\0');
  }

#endif
