/* --------------------------------------------------------
                    ntc_demo.c

     MCU : ATmega8, ATmega48/88/168/328p
     Takt: beliebig

     17.01.2021   R. Seelig
   -------------------------------------------------------- */
#include <avr/io.h>
#include <string.h>

#include "my_printf.h"
#include "uart_all.h"

#define printf    my_printf
#define delay     _delay_ms

/* --------------------------------------------------------
   my_putchar

   wird von my-printf / printf aufgerufen und hier muss
   eine Zeichenausgabefunktion angegeben sein, auf das
   printf dann schreibt !
   -------------------------------------------------------- */
void my_putchar(char ch)
{
  uart_putchar(ch);
}

/* --------------------------------------------------------
                        adc_10bit

     liest die "Ergebnisregister" des ADC aus. Hier
     wichtig: Niederwertiges Register ist zuerst zu
     lesen.

     Rueckgabe:
        10-Bit ADC-Wert des durch ADC's
   -------------------------------------------------------- */
unsigned int getadc_10bit (void)
{
  uint16_t adcwert,adch;

  adcwert= ADCL;                        // niederwertiges Register lesen
  adch= ADCH;                           // hoeherwertiges Register lesen
  adch = adch<<8;
  adcwert|= adch;
  return(adcwert);
}

/* --------------------------------------------------------
                        adc_init

     initialisiert den ADC und den Eingangsmultiplexer

     Uebergabe

     vref    : zu verwendende Referenzspannung
     channel : zu verwendenden Analogeingang (ADC0 .. ADC7)
               Hinweis: ADC6 und ADC7 nur in 32 pol.
                        Gehaeusen vorhanden
   -------------------------------------------------------- */
void adc_init(uint8_t vref, uint8_t channel)
{
/* <vref> bezeichnet die zu verwendende Spannungsreferenz
     0 : externe Referenz an Aref
     1 : +AVcc mit ext. Kondensator
     2 : reserviert
     3 : interne Referenz: 1,1V ATmega168 / 2,56V ATmega8

   <channel> bezeichnet den Kanal auf dem gemessen werden soll

   Spannungsreferenz ATMEGA328:

   ADMUX
   -----------------------------------------------------------------------
     Bit |  7    |   6   |   5   |   4   |   3   |   2   |   1   |   0   |
   -----------------------------------------------------------------------
    Name | REFS1 | REFS0 | ADLAR |   -   | MUX3  | MUX2  | MUX1  | MUX0  |


    REFS1  REFS0      Referenz
    --------------------------
      0      0        ext. Referenz an  Aref
      0      1        +AVcc mit ext. Kondensator
      1      0        reserviert
      1      1        Interne Spannungsreferenz: 1,1V (ATmega168) / 2,56V (ATmega8)

    ADLAR
    -----
    0 = rechtsbuendig, 1 = linksbuendig

     MUXx
    3 2 1 0
    -------------------------
    0 0 0 0   --
       ..       |->  Eingangskanaele ADC0 - ADC7
    0 1 1 1   --
    1 0 0 0   --
       ..       |->  reserviert
    1 1 0 1   --
    1 1 1 0          legt 1.1V an den ADC
    1 1 1 1          legt GND an den ADC


   ADCSRA
   -----------------------------------------------------------------------
     Bit |  7    |   6   |   5   |   4   |   3   |   2   |   1   |   0   |
   -----------------------------------------------------------------------
    Name | ADEN  | ADSC  | ADATE | ADIF  | ADIE  | ADPS2 | ADPS1 | ADPS0 |


    Scalermodus | ADPS2 | ADPS1 | ADPS0 | Teilerfaktor
    ---------------------------------------------------
         0      |   0  |   0   |   0   |       2
         1      |   0  |   0   |   1   |       2
         2      |   0  |   1   |   0   |       4
         3      |   0  |   1   |   1   |       8
         4      |   1  |   0   |   0   |      16
         5      |   1  |   0   |   1   |      32
         6      |   1  |   1   |   0   |      64
         7      |   1  |   1   |   1   |     128

*/

   ADMUX = (vref << 6) | channel;

   ADCSRA = (1 << ADEN) | (1 << ADSC) | (1 << ADATE) | 7;       // ADC enable, single, auto trigger
                                                                // Taktprescalermode 7 ( / 128)

   _delay_ms(1);
   ADCSRA &= ~(1 << ADSC);                                      // ADC single auf 0
}

/* -------------------------------------------------
     Lookup-table fuer NTC-Widerstand
     R25-Wert: 10.00 kOhm
     Pullup-Widerstand: 10.00 kOhm
     Materialkonstante beta: 3950
     Aufloesung des ADC: 10 Bit
     Einheit eines Tabellenwertes: 0.1 Grad Celcius
     Temperaturfehler der Tabelle: 0.5 Grad Celcius
   -------------------------------------------------*/
const int PROGMEM ntctable[] = {
  1269, 1016, 763, 621, 520, 439, 370, 308,
  250, 194, 139, 83, 22, -47, -132, -256,
  -380
};

/* -------------------------------------------------
                     ntc_gettemp

    zuordnen des Temperaturwertes aus gegebenem
    ADC-Wert.
   ------------------------------------------------- */
int ntc_gettemp(uint16_t adc_value)
{
  int p1,p2;

  // Stuetzpunkt vor und nach dem ADC Wert ermitteln.
  p1 = pgm_read_word(&(ntctable[ (adc_value >> 6)    ]));
  p2 = pgm_read_word(&(ntctable[ (adc_value >> 6) + 1]));

  // zwischen beiden Punkten interpolieren.
  return p1 - ( (p1-p2) * (adc_value & 0x003f) ) / 64;
}


/* --------------------------------------------------------
                             main
   -------------------------------------------------------- */
int main(void)
{
  uint8_t  count;
  uint8_t  ch;
  uint16_t mw;
  uint16_t temp;

  uart_init();
  adc_init(1,1);

  printf("\n\r------------------------------------");
  printf("\n\r        NTC-Demo (10k)");
  printf("\n\r------------------------------------\n\n\r");
  count= 0;

  printfkomma= 1;
  while(1)
  {
    mw= getadc_10bit ();
    temp= ntc_gettemp(mw);
    printf("\r adc: %d  ;  Temperatur: %k oC  ",mw, temp);
    _delay_ms(50);
  }
}
