/* -------------------------------------------------------
                      avr_rtc.c

     Softwareuhr, realisiert mittels Timer2 der durch
     einen 32,768 kHz Uhrenquartz getaktet ist

     MCU   :  ATMegaxx8
     Takt  :  Coretakt 8 MHz intern
              RTC-Takt 32,768 kHz Uhrenquarz

     Hinweis:

     Der Quarz wird wie ein "normaler" Quarz an die
     Anschluesse XTAL1, XTAL2 angeschlossen. Zieh-
     kondensatoren sollten im Bereich von 22pf bis 27pF
     liegen.

     Nach dem Start der Firmware kann es bis zu 7
     Sekunden dauern, bis der Taktgeber angeschwungen ist.


     04.05.2021  R. Seelig
   ------------------------------------------------------ */

#include "avr_rtc.h"

volatile uint8_t   std, min, sek, hsek;
volatile uint8_t   day, month;
volatile int       year;

/* --------------------------------------------------
                     getwtag

      Berechnet zu einem bestimmten Datum den
      Wochentag (nach Carl Friedrich Gauss):

      Rueckgabe: der Tag der Woche, beginnend ab
                 Sonntag. Sonntag entspricht 0.

      Bsp.:      02.06.2016   ( das ist ein Donnerstag )
      Rueckgabe: 5

   -------------------------------------------------- */
char getwtag(int day, int month, int year)
{
  int w_tag;

  if (month < 3)
  {
     month = month + 12;
     year--;
  }
  w_tag = (day+2*month + (3*month+3)/5 + year + year/4 - year/100 + year/400 + 1) % 7 ;
  return w_tag;
}


/* -----------------------------------------------------------
                    TIMER2 comp A ISR
                    Interruptvektor
   ----------------------------------------------------------- */
ISR (TIMER2_COMPA_vect)
{
  TCCR2B = TCCR2B;
  hsek++;
  if (hsek== 2)
  {
    sek++;
    hsek= 0;
  }
  if (sek==60)
  {
    sek= 0;
    min++;
    if (min==60)
    {
      min= 0;
      std++;
      if (std==24)
      {
        std= 0;

        // ab hier Kalenderfunktionen
        if (day< 28)                                            // Monatsende nicht erreicht
        {
          day++;
          return;
        }

        if (month== 2)
        {
          if ((year % 4)== 0)                                   // Schaltjahr
          {
            if (day==28)
            {
              day++;
              return;
            }
            else
            {
              month++;
              day= 1;
              return;
            }
          }
          else
          {
            month++;
            day= 1;
            return;
          }
        }

        // Monate mit 30 Tagen
        if ((month==4) || (month==6) || (month== 9) || (month== 11))     // April, Juni, September, November
        {
          if (day< 30)
          {
            day++;
            return;
          }
          else
          {
            day= 1;
            month++;
            return;
          }
        }

        // ab hier alle Monate mit 31 Tagen

        if ((month== 12) && (day== 31))                                // Feuerwerk, es ist Sylvester
        {
          day= 1;
          month= 1;
          year++;
          return;
        }

        if (day< 31)                                                  // Monatsende nicht erreicht
        {
          day++;
          return;
        }

        day= 1;                                                       // Monatsende der Monate mit 31 Tagen
        month++;
        return;

      }
    }
  }

  // Dummy-Zugriff
  TCCR2B = TCCR2B;

  // warten bis ein schreiben erfolgt ist
  while(ASSR & ((1 << TCN2UB) | (1 << OCR2AUB) | (1 << OCR2BUB) |
                (1 << TCR2AUB) | (1 << TCR2BUB)));

}

/* -----------------------------------------------------------
                        timer2_init

     initialisiert Timer2 fuer einen 32,768 kHz Uhren-
     quarz fuer einen Interruptintervall = 0,5 Sekunden
   ----------------------------------------------------------- */
void timer2_init(void)
{
  GTCCR |= (1 << TSM) | (1 << PSRASY);    // Timer stop, Prescaler Reset
  ASSR |= (1 << AS2);                     // Asynchron Mode
  TCCR2A = (1 << WGM21);                  // CTC Modus
  TCCR2B |= (1 << CS22) | (1 << CS21);    // Prescaler 256

  // 32768 / 256 / 0.5 = 64                Intervall = 0.5s

  OCR2A = 64 - 1;
  TIMSK2 |= (1 << OCIE2A);                // enable compare interrupt
  GTCCR &= ~(1 << TSM);                   // Timer starten

  sei();                                  // grundsaetzlich Interrupts zulassen
}

