/* -------------------------------------------------------
                          i2c_mega.c

     Softwaremodul zur Benutzung des  I2C Buses

     MCU   :  ATmega8, ATmega48 / 88 / 168 / 328p
     Takt  :

     27.01.2020  R. Seelig
   ------------------------------------------------------ */

#include <util/delay.h>
#include "i2c_mega.h"

/* ---------------------------------------------------------
                           i2c_init

      initialisiert den I2C-Bus
   --------------------------------------------------------- */
void i2c_init(void)
{
 TWBR = TWBR_val;
}

/* ---------------------------------------------------------
                           i2c_stop

      sendet das Stopsignal auf dem I2C Bus
   --------------------------------------------------------- */
void i2c_stop(void)
{
  // transmit STOP condition
  TWCR = (1<<TWINT) | (1<<TWEN) | (1<<TWSTO);}


/* ---------------------------------------------------------
                         i2c_startaddr

   startet den I2C-Bus und sendet Bauteileadresse.
   rwflag bestimmt, ob das Device beschrieben oder
   gelesen werden soll
   --------------------------------------------------------- */
uint8_t i2c_startaddr(uint8_t addr, uint8_t rwflag)
{
  addr = (addr | rwflag );

  // reset TWI control register
  TWCR = 0;
  // transmit START condition
  TWCR = (1<<TWINT) | (1<<TWSTA) | (1<<TWEN);
  // wait for end of transmission
  while( !(TWCR & (1<<TWINT)) );

  // check if the start condition was successfully transmitted
  if((TWSR & 0xF8) != TW_START){ return 1; }

  // load slave address into data register
  TWDR = addr;
  // start transmission of address
  TWCR = (1<<TWINT) | (1<<TWEN);
  // wait for end of transmission
  while( !(TWCR & (1<<TWINT)) );

  // check if the device has acknowledged the READ / WRITE mode
  if ((TW_STATUS & 0xF8) == 0x18) return 1;

  return 0;

}

/* ---------------------------------------------------------
                           i2c_write

      sendet den Wert in < value > auf dem I2C Bus.
      Rueckgabewert:
                     1 : ACK vom Slave gemeldet
                     0 : es wurde kein ACK eines Slaves
                         gesendet
   --------------------------------------------------------- */
uint8_t i2c_write(uint8_t value)
{
  // load data into data register
  TWDR = value;
  // start transmission of data
  TWCR = (1<<TWINT) | (1<<TWEN);
  // wait for end of transmission
  while( !(TWCR & (1<<TWINT)) );

  if( (TWSR & 0xF8) != TW_MT_DATA_ACK )
  {
    return 1;
  }

  return 0;
}

/* ---------------------------------------------------------
                     i2c_write16

   schreibt einen 16 Bit Integerwert mittels 2  Bytewrite
   Vorgaengen auf dem I2C Bus
  --------------------------------------------------------- */
void i2c_write16(uint16_t value)
{
  i2c_write(value >> 8);
  i2c_write(value & 0x00ff);
}

/* ---------------------------------------------------------
                           i2c_read

      liest I2C Bus und gibt ACK
      Rueckgabewert:
                     gelesener Wert
   --------------------------------------------------------- */
uint8_t i2c_read(void)
{
  // start TWI module and acknowledge data after reception
  TWCR = (1<<TWINT) | (1<<TWEN) | (1<<TWEA);
  // wait for end of transmission
  while( !(TWCR & (1<<TWINT)) );
  // return received data from TWDR
  return TWDR;}
/* ---------------------------------------------------------
                           i2c_read_nack

      liest I2C Bus und gibt keinen ACK nach lesen
      Rueckgabewert:
                     gelesener Wert
   --------------------------------------------------------- */
uint8_t i2c_read_nack(void)
{
  // start receiving without acknowledging reception
  TWCR = (1<<TWINT) | (1<<TWEN);
  // wait for end of transmission
  while( !(TWCR & (1<<TWINT)) );
  // return received data from TWDR
  return TWDR;
}

