/* -----------------------------------------------------------
                         max7219_dot8x8_4.c

   Softwaremodul fuer 4-fach kaskatierte 8x8 DOT-Matrixanzeige

   Benoetigte Hardware;

            - 4 Sueck 8x8 LED Matrix mit MAX7219 IC

   MCU   :  alle AVR
   Takt  :  intern oder extern


   23.03.2022 by R. Seelig
   ----------------------------------------------------------- */

#include "max7219_dot8x8_4.h"

  // "Framebuffer" der ein komplettes 32x8 Pixelbitmap aufnehmen kann
  uint8_t fbuf[32];

  uint8_t fontnr = 0;                     //  0 : 5x7  Font
                                          //  1 : 8x8  Font
  uint8_t fontsizex = 6;

  uint8_t aktxp = 0;                      // Beinhaltet die aktuelle Position des Textcursors in X-Achse
  uint8_t aktyp = 0;                      // dto. fuer die Y-Achse

  extern  const uint8_t PROGMEM font5x7[][5];
  extern  const uint8_t PROGMEM font8x8[][8];


/* -----------------------------------------------------------
     fb_clr

     loescht den Framebuffer
   ----------------------------------------------------------- */
void fb_clr(void)
{
  uint8_t i;

  for (i= 0; i< 32; i++)
  {
    fbuf[i]= 0;
  }
}

/* -----------------------------------------------------------
     serout

     serielle Datenausgabe mittels Bitbanging, MSB zuerst
   ----------------------------------------------------------- */
void serout(uint8_t data)
{
  uint8_t i, val;

  for(i= 8; i> 0; i--)
  {
    val = (data & (1 << (i-1)) ) >> (i - 1);
    m7219_clkclr();
    if(val)
      m7219_dinset();
    else
      m7219_dinclr();
    m7219_clkset();
  }
}


/* -----------------------------------------------------------
     m7219_send_nl

     sendet ein Byte an ein Register. Es wird KEIN Load-Store
     durchgefuehrt. Somit werden weiter ankommende Bytes an
     das jeweils nachfolgende Schieberegister des MAX7219
     gesendet.

        reg   : zu beschreibendes Register des MAX7219
        data  : Datum, das seriell gesendet wird

   ----------------------------------------------------------- */
void m7219_send_nl(uint8_t reg, uint8_t data)
{
  serout(reg);
  serout(data);
}

/* -----------------------------------------------------------
     m7219_send4

     sendet 4 Bytes in kaskadierten MAX7219 in jeweils die-
     selbe ROW

        row   : zu beschreibendes Register des MAX7219
        *data : Zeiger auf den Speicherbereich der 4 Bytes

   ----------------------------------------------------------- */
void m7219_send4(uint8_t row, uint8_t *data)
{
  uint8_t i;

  m7219_loadclr();

  data += 3;
  for (i= 0; i< 4; i++)
  {
    serout(row+1);
    serout(*data);
    data--;
  }

  m7219_loadset();
}


/* -----------------------------------------------------------
     m7219_col32

     sendet die 4 Bytes eines uint32_t in eine ROW

        row   : zu beschreibendes Register des MAX7219
        data  : 32-Bit Wert, der das Bitmap einer kompletten
                row enthaelt
   ----------------------------------------------------------- */
void m7219_col32(uint8_t reg, uint32_t data)
{
  m7219_loadclr();

  serout(reg+1);
  serout((data >> 24l) & 0xff);

  serout(reg+1);
  serout((data >> 16l) & 0xff);

  serout(reg+1);
  serout((data >> 8l) & 0xff);

  serout(reg+1);
  serout(data & 0xff);

  m7219_loadset();
}

/* -----------------------------------------------------------
     m7219_clrscr

     loescht die gesamte Matrixanzeige
   ----------------------------------------------------------- */
void m7219_clrscr()
{
  uint8_t i;

  for(i= 0; i< 8; i++)
  {
    m7219_col32(i, 0);
  }

  fb_clr();
}


/* -----------------------------------------------------------
     m7219_init4

     Setzt die Anschluesse an die der MAX7219 angeschlossen ist
     als Ausgaenge und intitalisiert die Register des ICs
   ----------------------------------------------------------- */
void m7219_init4()
{
  uint8_t i;

  m7219_init_con();

  m7219_dinclr();
  m7219_clkclr();
  m7219_loadclr();


  m7219_loadclr();
  for (i= 0; i< 4; i++) { m7219_send_nl(0x0b, 0x07); }        // Scanlimit
  m7219_loadset();

  m7219_loadclr();
  for (i= 0; i< 4; i++) { m7219_send_nl(0x09, 0x00); }        // LED-Matrix, keine 7 Segment
  m7219_loadset();

  m7219_loadclr();
  for (i= 0; i< 4; i++) { m7219_send_nl(0x0c, 0x01); }       // Power an
  m7219_loadset();

  m7219_loadclr();
  for (i= 0; i< 4; i++) { m7219_send_nl(0x0f, 0x00); }       // kein Displaytest
  m7219_loadset();

  m7219_loadclr();
  for (i= 0; i< 4; i++) { m7219_send_nl(0x0a, 0x02); }       // Intensitaet
  m7219_loadset();


  m7219_clrscr();
}

/* -----------------------------------------------------------
     m7219_setfb

     zeigt den Inhalt des Framebuffers auf der DOT Matrix-
     anzeige an
   ----------------------------------------------------------- */
void m7219_setfb(void)
{
  uint8_t i, cx= 0;

  for (i= 0; i< 8; i++)
  {
    m7219_send4(i, &fbuf[i*4]);
  }
}

/* -----------------------------------------------------------
     fb_putpixel

     setzt / loescht ein einzelnes Pixel im Framebuffer mit
     der Farbe f

       x,y : Koordinate des Pixel
         f : "Farbe" des Pixels, 1 = LED an, 0 = LED aus
   ----------------------------------------------------------- */
void fb_putpixel(int16_t x, int16_t y, uint8_t f)
{
  int16_t addr, bp, mask;

  if ((y<0) || (y>7)) return;
  if ((x<0) || (x>31)) return;
  y= 7-y;
  addr= (x / 8) + (y * 4);
  if ((addr < 0) || (addr > 31)) return;
  bp= x % 8;
  mask= (1 << bp);

  if (f)
  {
    fbuf[addr] |= mask;
  }
  else
  {
    fbuf[addr] &= ~mask;
  }
}

/* -------------------------------------------------------------
     fb_line

     Zeichnet eine Linie von den Koordinaten x0,y0 zu x1,y1
     mit der angegebenen Farbe

        x0,y0 : Koordinate linke obere Ecke
        x1,y1 : Koordinate rechte untere Ecke
        color : 1: LED leuchtet, 0: LED aus

     Linienalgorithmus nach Bresenham (www.wikipedia.org)

   ------------------------------------------------------------- */
void fb_line(int x0, int y0, int x1, int y1, uint8_t color)
{

  //    Linienalgorithmus nach Bresenham (www.wikipedia.org)

  int dx =  abs(x1-x0), sx = x0<x1 ? 1 : -1;
  int dy = -abs(y1-y0), sy = y0<y1 ? 1 : -1;
  int err = dx+dy, e2;                                     /* error value e_xy */

  for(;;)
  {

    fb_putpixel(x0,y0,color);
    if (x0==x1 && y0==y1) break;
    e2 = 2*err;
    if (e2 > dy) { err += dy; x0 += sx; }                  /* e_xy+e_x > 0 */
    if (e2 < dx) { err += dx; y0 += sy; }                  /* e_xy+e_y < 0 */
  }
}

/*  ---------------------------------------------------------
                            gotoxy

       legt die naechste Textausgabeposition im Framebuffer
       fest. Koordinaten 0,0 bezeichnet linke obere
       Position
    --------------------------------------------------------- */
void gotoxy(uint8_t x, uint8_t y)
{
  aktxp= x; aktyp= y;
}


/*  ---------------------------------------------------------
                           setfont

      legt Schriftstil fuer die Zeichenausgabe fest.

      fnr== 0  => Font 5x7
      fnr== 1  => Font 8x8
    --------------------------------------------------------- */
void setfont(uint8_t fnr)
{
  if (fnr > 1) { fontnr= 0; return; };
  fontnr= fnr;
  if (fnr == 0) { fontsizex= 6; return; };
  if (fnr == 1) { fontsizex= 8; return; };
  return ;
}


/* -----------------------------------------------------------
     fb_putchar

     gibt ein Zeichen im Framebuffer aus

       x,y : Koordinate, an der das Zeichen ausgegeben werden
             soll
        ch : auszugebendes Zeichen
   ----------------------------------------------------------- */
void fb_putchar(uint8_t x, uint8_t y, uint8_t ch)
{
  uint8_t xo, yo;
  uint8_t rb, rt;

  for (xo= 0; xo < fontsizex; xo++)
  {
    if (fontnr) rb= pgm_read_byte(&(font8x8[ch-32][xo]));
           else rb= pgm_read_byte(&(font5x7[ch-32][xo]));
    if ((xo== 5) && (fontsizex== 6)) rb= 0;

    for (yo= 0; yo < 8; yo++)
    {
      if (rb & 0x01) fb_putpixel(x+xo, y+yo, 1);
                else fb_putpixel(x+xo, y+yo, 0);
      rb= rb >> 1;
    }
  }
}

/* --------------------------------------------------------
   m7219_tx_scrollin

   scrollt einen Text von rechts nach links auf der
   DOT-Matrix Anzeige
   -------------------------------------------------------- */
void m7219_tx_scrollin(int dt, txary *tx)
{
  int i,w;

  for (i= 31; i> -1; i--)
  {
    gotoxy(i,0);
    fb_clr();
    printf("%s",tx);
    m7219_setfb();
    for (w= 0; w< dt; w++) { delay(1);}
  }
}

#if (font5x7_enable == 1)

  /* Bitmaps des Ascii-Zeichensatzes
     ein Smily wuerde so aussehen:
        { 0x36, 0x46, 0x40, 0x46, 0x36 }  // Smiley

     ein grosses R ist folgendermassen definiert:

     { 0x7F, 0x09, 0x19, 0x29, 0x46 },   // R

     . x x x x x x x
     . . . . x . . x
     . . . x x . . x
     . . x . x . . x
     . x . . . x x .

  */

  const uint8_t PROGMEM font5x7[][5] =
  {
    { 0x00, 0x00, 0x00, 0x00, 0x00 },   // space
    { 0x00, 0x00, 0x2f, 0x00, 0x00 },   // !
    { 0x00, 0x07, 0x00, 0x07, 0x00 },   // "
    { 0x14, 0x7f, 0x14, 0x7f, 0x14 },   // #
    { 0x24, 0x2a, 0x7f, 0x2a, 0x12 },   // $
    { 0xc4, 0xc8, 0x10, 0x26, 0x46 },   // %
    { 0x36, 0x49, 0x55, 0x22, 0x50 },   // &
    { 0x00, 0x05, 0x03, 0x00, 0x00 },   // '
    { 0x00, 0x1c, 0x22, 0x41, 0x00 },   // (
    { 0x00, 0x41, 0x22, 0x1c, 0x00 },   // )
    { 0x14, 0x08, 0x3E, 0x08, 0x14 },   // *
    { 0x08, 0x08, 0x3E, 0x08, 0x08 },   // +
    { 0x00, 0x00, 0x50, 0x30, 0x00 },   // ,
    { 0x10, 0x10, 0x10, 0x10, 0x10 },   // -
    { 0x00, 0x60, 0x60, 0x00, 0x00 },   // .
    { 0x20, 0x10, 0x08, 0x04, 0x02 },   // /
    { 0x3E, 0x51, 0x49, 0x45, 0x3E },   // 0
    { 0x00, 0x42, 0x7F, 0x40, 0x00 },   // 1
    { 0x42, 0x61, 0x51, 0x49, 0x46 },   // 2
    { 0x21, 0x41, 0x45, 0x4B, 0x31 },   // 3
    { 0x18, 0x14, 0x12, 0x7F, 0x10 },   // 4
    { 0x27, 0x45, 0x45, 0x45, 0x39 },   // 5
    { 0x3C, 0x4A, 0x49, 0x49, 0x30 },   // 6
    { 0x01, 0x71, 0x09, 0x05, 0x03 },   // 7
    { 0x36, 0x49, 0x49, 0x49, 0x36 },   // 8
    { 0x06, 0x49, 0x49, 0x29, 0x1E },   // 9
    { 0x00, 0x36, 0x36, 0x00, 0x00 },   // :
    { 0x00, 0x56, 0x36, 0x00, 0x00 },   // ;
    { 0x08, 0x14, 0x22, 0x41, 0x00 },   // <
    { 0x14, 0x14, 0x14, 0x14, 0x14 },   // =
    { 0x00, 0x41, 0x22, 0x14, 0x08 },   // >
    { 0x02, 0x01, 0x51, 0x09, 0x06 },   // ?
    { 0x32, 0x49, 0x59, 0x51, 0x3E },   // @
    { 0x7E, 0x11, 0x11, 0x11, 0x7E },   // A
    { 0x7F, 0x49, 0x49, 0x49, 0x36 },   // B
    { 0x3E, 0x41, 0x41, 0x41, 0x22 },   // C
    { 0x7F, 0x41, 0x41, 0x22, 0x1C },   // D
    { 0x7F, 0x49, 0x49, 0x49, 0x41 },   // E
    { 0x7F, 0x09, 0x09, 0x09, 0x01 },   // F
    { 0x3E, 0x41, 0x49, 0x49, 0x7A },   // G
    { 0x7F, 0x08, 0x08, 0x08, 0x7F },   // H
    { 0x00, 0x41, 0x7F, 0x41, 0x00 },   // I
    { 0x20, 0x40, 0x41, 0x3F, 0x01 },   // J
    { 0x7F, 0x08, 0x14, 0x22, 0x41 },   // K
    { 0x7F, 0x40, 0x40, 0x40, 0x40 },   // L
    { 0x7F, 0x02, 0x0C, 0x02, 0x7F },   // M
    { 0x7F, 0x04, 0x08, 0x10, 0x7F },   // N
    { 0x3E, 0x41, 0x41, 0x41, 0x3E },   // O
    { 0x7F, 0x09, 0x09, 0x09, 0x06 },   // P
    { 0x3E, 0x41, 0x51, 0x21, 0x5E },   // Q
    { 0x7F, 0x09, 0x19, 0x29, 0x46 },   // R
    { 0x46, 0x49, 0x49, 0x49, 0x31 },   // S
    { 0x01, 0x01, 0x7F, 0x01, 0x01 },   // T
    { 0x3F, 0x40, 0x40, 0x40, 0x3F },   // U
    { 0x1F, 0x20, 0x40, 0x20, 0x1F },   // V
    { 0x3F, 0x40, 0x38, 0x40, 0x3F },   // W
    { 0x63, 0x14, 0x08, 0x14, 0x63 },   // X
    { 0x07, 0x08, 0x70, 0x08, 0x07 },   // Y
    { 0x61, 0x51, 0x49, 0x45, 0x43 },   // Z
    { 0x00, 0x7F, 0x41, 0x41, 0x00 },   // [
    { 0x55, 0x2A, 0x55, 0x2A, 0x55 },   // "Yen"
    { 0x00, 0x41, 0x41, 0x7F, 0x00 },   // ]
    { 0x04, 0x02, 0x01, 0x02, 0x04 },   // ^
    { 0x40, 0x40, 0x40, 0x40, 0x40 },   // _
    { 0x00, 0x01, 0x02, 0x04, 0x00 },   // '
    { 0x20, 0x54, 0x54, 0x54, 0x78 },   // a
    { 0x7F, 0x48, 0x44, 0x44, 0x38 },   // b
    { 0x38, 0x44, 0x44, 0x44, 0x20 },   // c
    { 0x38, 0x44, 0x44, 0x48, 0x7F },   // d
    { 0x38, 0x54, 0x54, 0x54, 0x18 },   // e
    { 0x08, 0x7E, 0x09, 0x01, 0x02 },   // f
    { 0x0C, 0x52, 0x52, 0x52, 0x3E },   // g
    { 0x7F, 0x08, 0x04, 0x04, 0x78 },   // h
    { 0x00, 0x44, 0x7D, 0x40, 0x00 },   // i
    { 0x20, 0x40, 0x44, 0x3D, 0x00 },   // j
    { 0x7F, 0x10, 0x28, 0x44, 0x00 },   // k
    { 0x00, 0x41, 0x7F, 0x40, 0x00 },   // l
    { 0x7C, 0x04, 0x18, 0x04, 0x78 },   // m
    { 0x7C, 0x08, 0x04, 0x04, 0x78 },   // n
    { 0x38, 0x44, 0x44, 0x44, 0x38 },   // o
    { 0x7C, 0x14, 0x14, 0x14, 0x08 },   // p
    { 0x08, 0x14, 0x14, 0x18, 0x7C },   // q
    { 0x7C, 0x08, 0x04, 0x04, 0x08 },   // r
    { 0x48, 0x54, 0x54, 0x54, 0x20 },   // s
    { 0x04, 0x3F, 0x44, 0x40, 0x20 },   // t
    { 0x3C, 0x40, 0x40, 0x20, 0x7C },   // u
    { 0x1C, 0x20, 0x40, 0x20, 0x1C },   // v
    { 0x3C, 0x40, 0x30, 0x40, 0x3C },   // w
    { 0x44, 0x28, 0x10, 0x28, 0x44 },   // x
    { 0x0C, 0x50, 0x50, 0x50, 0x3C },   // y
    { 0x44, 0x64, 0x54, 0x4C, 0x44 },   // z
    // Zeichen vom Ascii-Satz abweichend
    { 0x3E, 0x7F, 0x7F, 0x3E, 0x00 },   // Zeichen 123 : ausgefuelltes Oval
    { 0x06, 0x09, 0x09, 0x06, 0x00 },   // Zeichen 124 : hochgestelltes kleines o (fuer Gradzeichen);
    { 0x01, 0x01, 0x01, 0x01, 0x01 },   // Zeichen 125 : Strich in der obersten Reihe
    { 0x00, 0x1D, 0x15, 0x17, 0x00 },   // Zeichen 126 : "hoch 2"
    { 0x47, 0x35, 0x7f, 0x56, 0x71 }    // Zeichen 127 : %

  };
#else
  // Dummy-Array

  const uint8_t PROGMEM font5x7[][5] = {{ 0 }};

#endif


#if (font8x8_enable == 1)

  /* Bitmaps des Ascii-Zeichensatzes

     ein grosses P ist folgendermassen definiert:

     { 0x7f, 0x7f, 0x09, 0x09, 0x09, 0x0f, 0x06, 0x00 },    // Ascii 80 = 'P'

       x x x x x x x
       x x x x x x x
             x     x
             x     x
             x     x
             x x x x
               x x
  */

  const uint8_t PROGMEM font8x8[][8] =
  {
    { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },    // Ascii 32 = ' '
    { 0x00, 0x00, 0x00, 0x5f, 0x5f, 0x00, 0x00, 0x00 },    // Ascii 33 = '!'
    { 0x00, 0x07, 0x07, 0x00, 0x07, 0x07, 0x00, 0x00 },    // Ascii 34 = '"'
    { 0x14, 0x7f, 0x7f, 0x14, 0x7f, 0x7f, 0x14, 0x00 },    // Ascii 35 = '#'
    { 0x00, 0x24, 0x2a, 0x7f, 0x7f, 0x2a, 0x12, 0x00 },    // Ascii 36 = '$'
    { 0x00, 0x46, 0x66, 0x30, 0x18, 0x0c, 0x66, 0x62 },    // Ascii 37 = '%'
    { 0x00, 0x30, 0x7a, 0x4f, 0x5d, 0x37, 0x7a, 0x48 },    // Ascii 38 = '&'
    { 0x00, 0x00, 0x04, 0x07, 0x03, 0x00, 0x00, 0x00 },    // Ascii 39 = '''
    { 0x00, 0x00, 0x1c, 0x3e, 0x63, 0x41, 0x00, 0x00 },    // Ascii 40 = '('
    { 0x00, 0x00, 0x41, 0x63, 0x3e, 0x1c, 0x00, 0x00 },    // Ascii 41 = ')'
    { 0x08, 0x2a, 0x3e, 0x1c, 0x1c, 0x3e, 0x2a, 0x08 },    // Ascii 42 = '*'
    { 0x00, 0x08, 0x08, 0x3e, 0x3e, 0x08, 0x08, 0x00 },    // Ascii 43 = '+'
    { 0x00, 0x00, 0x00, 0x60, 0x60, 0x00, 0x00, 0x00 },    // Ascii 44 = ','
    { 0x00, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x00 },    // Ascii 45 = '-'
    { 0x00, 0x00, 0x00, 0x60, 0x60, 0x00, 0x00, 0x00 },    // Ascii 46 = '.'
    { 0x60, 0x30, 0x18, 0x0c, 0x06, 0x03, 0x01, 0x00 },    // Ascii 47 = '/'
    { 0x3e, 0x7f, 0x51, 0x49, 0x45, 0x7f, 0x3e, 0x00 },    // Ascii 48 = '0'
    { 0x00, 0x00, 0x02, 0x7f, 0x7f, 0x00, 0x00, 0x00 },    // Ascii 49 = '1'
    { 0x00, 0x42, 0x63, 0x71, 0x59, 0x4f, 0x46, 0x00 },    // Ascii 50 = '2'
    { 0x00, 0x22, 0x63, 0x49, 0x49, 0x7f, 0x36, 0x00 },    // Ascii 51 = '3'
    { 0x18, 0x1c, 0x16, 0x13, 0x7f, 0x7f, 0x10, 0x00 },    // Ascii 52 = '4'
    { 0x00, 0x27, 0x67, 0x45, 0x45, 0x7d, 0x39, 0x00 },    // Ascii 53 = '5'
    { 0x00, 0x3e, 0x7f, 0x49, 0x49, 0x7b, 0x32, 0x00 },    // Ascii 54 = '6'
    { 0x00, 0x01, 0x01, 0x71, 0x79, 0x0f, 0x07, 0x00 },    // Ascii 55 = '7'
    { 0x00, 0x36, 0x7f, 0x49, 0x49, 0x7f, 0x36, 0x00 },    // Ascii 56 = '8'
    { 0x00, 0x06, 0x4f, 0x69, 0x39, 0x1f, 0x0e, 0x00 },    // Ascii 57 = '9'
    { 0x00, 0x00, 0x00, 0x6c, 0x6c, 0x00, 0x00, 0x00 },    // Ascii 58 = ':'
    { 0x00, 0x00, 0x01, 0x6d, 0x6c, 0x00, 0x00, 0x00 },    // Ascii 59 = ';'
    { 0x00, 0x08, 0x1c, 0x36, 0x63, 0x41, 0x00, 0x00 },    // Ascii 60 = '<'
    { 0x00, 0x24, 0x24, 0x24, 0x24, 0x24, 0x24, 0x00 },    // Ascii 61 = '='
    { 0x00, 0x41, 0x63, 0x36, 0x1c, 0x08, 0x00, 0x00 },    // Ascii 62 = '>'
    { 0x00, 0x02, 0x03, 0x51, 0x59, 0x0f, 0x06, 0x00 },    // Ascii 63 = '?'
    { 0x3e, 0x7f, 0x41, 0x5d, 0x5d, 0x5f, 0x1e, 0x00 },    // Ascii 64 = '@'
    { 0x7c, 0x7e, 0x13, 0x11, 0x13, 0x7e, 0x7c, 0x00 },    // Ascii 65 = 'A'
    { 0x7f, 0x7f, 0x49, 0x49, 0x49, 0x7f, 0x36, 0x00 },    // Ascii 66 = 'B'
    { 0x1c, 0x3e, 0x63, 0x41, 0x41, 0x63, 0x22, 0x00 },    // Ascii 67 = 'C'
    { 0x7f, 0x7f, 0x41, 0x41, 0x63, 0x3e, 0x1c, 0x00 },    // Ascii 68 = 'D'
    { 0x7f, 0x7f, 0x49, 0x49, 0x49, 0x41, 0x41, 0x00 },    // Ascii 69 = 'E'
    { 0x7f, 0x7f, 0x09, 0x09, 0x09, 0x01, 0x01, 0x00 },    // Ascii 70 = 'F'
    { 0x1c, 0x3e, 0x63, 0x41, 0x51, 0x73, 0x72, 0x00 },    // Ascii 71 = 'G'
    { 0x7f, 0x7f, 0x08, 0x08, 0x08, 0x7f, 0x7f, 0x00 },    // Ascii 72 = 'H'
    { 0x00, 0x41, 0x7f, 0x7f, 0x41, 0x00, 0x00, 0x00 },    // Ascii 73 = 'I'
    { 0x30, 0x70, 0x40, 0x40, 0x40, 0x7f, 0x3f, 0x00 },    // Ascii 74 = 'J'
    { 0x7f, 0x7f, 0x09, 0x1d, 0x36, 0x63, 0x41, 0x00 },    // Ascii 75 = 'K'
    { 0x7f, 0x7f, 0x40, 0x40, 0x40, 0x40, 0x40, 0x00 },    // Ascii 76 = 'L'
    { 0x7f, 0x7f, 0x06, 0x0c, 0x06, 0x7f, 0x7f, 0x00 },    // Ascii 77 = 'M'
    { 0x7f, 0x7f, 0x06, 0x0c, 0x18, 0x7f, 0x7f, 0x00 },    // Ascii 78 = 'N'
    { 0x1c, 0x3e, 0x63, 0x41, 0x63, 0x3e, 0x1c, 0x00 },    // Ascii 79 = 'O'
    { 0x7f, 0x7f, 0x09, 0x09, 0x09, 0x0f, 0x06, 0x00 },    // Ascii 80 = 'P'
    { 0x1c, 0x3e, 0x63, 0x51, 0x33, 0x6e, 0x5c, 0x00 },    // Ascii 81 = 'Q'
    { 0x7f, 0x7f, 0x09, 0x09, 0x19, 0x7f, 0x66, 0x00 },    // Ascii 82 = 'R'
    { 0x26, 0x6f, 0x49, 0x49, 0x49, 0x7b, 0x32, 0x00 },    // Ascii 83 = 'S'
    { 0x01, 0x01, 0x7f, 0x7f, 0x01, 0x01, 0x00, 0x00 },    // Ascii 84 = 'T'
    { 0x3f, 0x7f, 0x40, 0x40, 0x40, 0x7f, 0x3f, 0x00 },    // Ascii 85 = 'U'
    { 0x1f, 0x3f, 0x60, 0x40, 0x60, 0x3f, 0x1f, 0x00 },    // Ascii 86 = 'V'
    { 0x7f, 0x7f, 0x30, 0x18, 0x30, 0x7f, 0x7f, 0x00 },    // Ascii 87 = 'W'
    { 0x63, 0x77, 0x1c, 0x08, 0x1c, 0x77, 0x63, 0x00 },    // Ascii 88 = 'X'
    { 0x00, 0x07, 0x0f, 0x78, 0x78, 0x0f, 0x07, 0x00 },    // Ascii 89 = 'Y'
    { 0x41, 0x61, 0x71, 0x59, 0x4d, 0x47, 0x43, 0x00 },    // Ascii 90 = 'Z'
    { 0x00, 0x00, 0x7f, 0x7f, 0x41, 0x41, 0x00, 0x00 },    // Ascii 91 = '['
    { 0x01, 0x03, 0x06, 0x0c, 0x18, 0x30, 0x60, 0x00 },    // Ascii 92 = '\'
    { 0x00, 0x00, 0x41, 0x41, 0x7f, 0x7f, 0x00, 0x00 },    // Ascii 93 = ']'
    { 0x00, 0x08, 0x0c, 0x06, 0x03, 0x06, 0x0c, 0x08 },    // Ascii 94 = '^'
    { 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80 },    // Ascii 95 = '_'
    { 0x00, 0x00, 0x01, 0x03, 0x06, 0x04, 0x00, 0x00 },    // Ascii 96 = '`'
    { 0x20, 0x74, 0x54, 0x54, 0x54, 0x7c, 0x78, 0x00 },    // Ascii 97 = 'a'
    { 0x7f, 0x7f, 0x44, 0x44, 0x44, 0x7c, 0x38, 0x00 },    // Ascii 98 = 'b'
    { 0x38, 0x7c, 0x44, 0x44, 0x44, 0x6c, 0x28, 0x00 },    // Ascii 99 = 'c'
    { 0x38, 0x7c, 0x44, 0x44, 0x44, 0x7f, 0x7f, 0x00 },    // Ascii 100 = 'd'
    { 0x38, 0x7c, 0x54, 0x54, 0x54, 0x5c, 0x18, 0x00 },    // Ascii 101 = 'e'
    { 0x08, 0x7e, 0x7f, 0x09, 0x09, 0x03, 0x02, 0x00 },    // Ascii 102 = 'f'
    { 0x98, 0xbc, 0xa4, 0xa4, 0xa4, 0xfc, 0x7c, 0x00 },    // Ascii 103 = 'g'
    { 0x7f, 0x7f, 0x04, 0x04, 0x04, 0x7c, 0x78, 0x00 },    // Ascii 104 = 'h'
    { 0x00, 0x00, 0x00, 0x7d, 0x7d, 0x00, 0x00, 0x00 },    // Ascii 105 = 'i'
    { 0x40, 0xc0, 0x80, 0x80, 0xfd, 0x7d, 0x00, 0x00 },    // Ascii 106 = 'j'
    { 0x7f, 0x7f, 0x10, 0x10, 0x38, 0x6c, 0x44, 0x00 },    // Ascii 107 = 'k'
    { 0x00, 0x00, 0x00, 0x7f, 0x7f, 0x00, 0x00, 0x00 },    // Ascii 108 = 'l'
    { 0x78, 0x7c, 0x0c, 0x18, 0x0c, 0x7c, 0x78, 0x00 },    // Ascii 109 = 'm'
    { 0x7c, 0x7c, 0x04, 0x04, 0x04, 0x7c, 0x78, 0x00 },    // Ascii 110 = 'n'
    { 0x38, 0x7c, 0x44, 0x44, 0x44, 0x7c, 0x38, 0x00 },    // Ascii 111 = 'o'
    { 0xfc, 0xfc, 0x24, 0x24, 0x24, 0x3c, 0x18, 0x00 },    // Ascii 112 = 'p'
    { 0x18, 0x3c, 0x24, 0x24, 0x24, 0xfc, 0xfc, 0x00 },    // Ascii 113 = 'q'
    { 0x7c, 0x7c, 0x04, 0x04, 0x04, 0x0c, 0x08, 0x00 },    // Ascii 114 = 'r'
    { 0x48, 0x5c, 0x54, 0x54, 0x54, 0x74, 0x20, 0x00 },    // Ascii 115 = 's'
    { 0x04, 0x3f, 0x7f, 0x44, 0x44, 0x64, 0x20, 0x00 },    // Ascii 116 = 't'
    { 0x3c, 0x7c, 0x40, 0x40, 0x40, 0x7c, 0x7c, 0x00 },    // Ascii 117 = 'u'
    { 0x1c, 0x3c, 0x60, 0x40, 0x60, 0x3c, 0x1c, 0x00 },    // Ascii 118 = 'v'
    { 0x3c, 0x7c, 0x60, 0x30, 0x60, 0x7c, 0x3c, 0x00 },    // Ascii 119 = 'w'
    { 0x44, 0x6c, 0x38, 0x10, 0x38, 0x6c, 0x44, 0x00 },    // Ascii 120 = 'x'
    { 0x9c, 0xbc, 0xa0, 0xa0, 0xa0, 0xfc, 0x7c, 0x00 },    // Ascii 121 = 'y'
    { 0x00, 0x44, 0x64, 0x74, 0x5c, 0x4c, 0x44, 0x00 },    // Ascii 122 = 'z'
    { 0x00, 0x08, 0x08, 0x3e, 0x77, 0x41, 0x41, 0x00 },    // Ascii 123 = '{'
    { 0x00, 0x00, 0x00, 0x77, 0x77, 0x00, 0x00, 0x00 },    // Ascii 124 = '|'
    { 0x00, 0x41, 0x41, 0x77, 0x3e, 0x08, 0x08, 0x00 },    // Ascii 125 = '}'
    { 0x00, 0x02, 0x01, 0x01, 0x02, 0x02, 0x01, 0x00 },    // Ascii 126 = '~'
    { 0x70, 0x78, 0x4c, 0x46, 0x46, 0x4c, 0x78, 0x70 },    // Ascii 127 = ''

    { 0x3c, 0x42, 0x81, 0x81, 0x81, 0x42, 0x3c, 0x00 },    // Ascii 128 abweichend: rundes o
    { 0x3c, 0x42, 0xbd, 0xbd, 0xbd, 0x42, 0x3c, 0x00 },    // Ascii 129 abweichend: ausgefuelltes o
    { 0x00, 0x00, 0x0e, 0x11, 0x11, 0x11, 0x0e, 0x00 },    // Ascii 130 abweichend: hochgestelltes o
    { 0x47, 0x65, 0x37, 0x18, 0x0c, 0x76, 0x53, 0x71 }     // Ascii 131 abweichend: %


  };
#else
  // Dummy-Array

  const uint8_t PROGMEM font8x8[][8] = {{ 0 }};

#endif

