/* ---------------------------------------------------------------
                                shiftout.c

     Universelles serielle Schiebeausgabe mittels Bitbanging

     In der globalen struct Variable
                       "shiftparam"
     werden die Schiebeparameter festgelegt:

            shiftparam.msbfirst   : 1 = MSB zuerst (default)
                                    0 = LSB zuerst
            shiftparam.clkol      : 1 = Taktleitung ist im (default)
                                        Ruhezustand 0
                                    0 = Ruhezustand 1
            shiftparam.clkcycles  : Anzahl Wiederholungen von
                                    _delay_us(1) waehrend des
                                    Taktimpulses
                                    Defaultwert = 1

     Schiebetaktfrequenz bei F_CPU= 16 MHz


     shiftparam.clkcycles |   0     |     1   |    10    |  100  |
     -------------------------------------------------------------
     ungefaehre Taktfreq. | 166 kHz |  85 kHz | 17.5 kHz | 2 kHz |


     MCU     : alle AVR
     F_CPU   :

     04.12.2019   R.Seelig
   --------------------------------------------------------------- */

#include "shiftout.h"

// Nimmt Optionsparameter fuer serielles Schieben auf
struct shiftsettings shiftparam;


/* ----------------------------------------------------------
                       shiftout_clkdelay

   Verzoegerungsfunktion zur Erzeugung von Takt und Strobe-
   impuls
   ---------------------------------------------------------- */
void shiftout_clkdelay(void)
{
  volatile uint16_t cx;

  if (shiftparam.clkcycles)
  {
    for (cx= 0; cx < shiftparam.clkcycles; cx++)
      _delay_us(1);
  }
}

/* ----------------------------------------------------------
                       shiftout_clkpuls

   erzeugt einen Hi-Taktimpuls, CLK-Leitung ist Standard-
   maessig auf 0
   ---------------------------------------------------------- */
void shiftout_clkpuls(void)
// Schieberegister Taktimpuls
{
  if (shiftparam.clkpol)
  {
    shiftout_clkdelay();
    shift_clk_set();
    shiftout_clkdelay();
    shift_clk_clr();
  }
  else
  {
    shiftout_clkdelay();
    shift_clk_clr();
    shiftout_clkdelay();
    shift_clk_set();

  }
}

/* ----------------------------------------------------------
                      shiftout_msbfirst

   schiebt das Byte in "value" mit MSB zuerst seriell aus
   ---------------------------------------------------------- */
void shiftout_msbfirst(uint8_t value)
{
  uint8_t mask, b;

  mask= 0x80;                                    // Maske fuer MSB setzen

  for (b= 0; b< 8; b++)
  {
    // Byte ins Schieberegister schieben, MSB zuerst
    if (mask & value) shift_bit_set();           // 1 oder 0 entsprechend Wert setzen
                 else shift_bit_clr();

    shiftout_clkpuls();                         // ... Puls erzeugen
    mask= mask >> 1;                             // Maske auf naechstes Bit setzen
  }
}

/* ----------------------------------------------------------
                      shiftout_lsbfirst

   schiebt das Byte in "value" mit LSB zuerst seriell aus
   ---------------------------------------------------------- */
void shiftout_lsbfirst(uint8_t value)
{
  uint8_t mask, b;

  mask= 0x01;                                    // Maske fuer LSB setzen

  for (b= 0; b< 8; b++)
  {
    // Byte ins Schieberegister schieben, MSB zuerst
    if (mask & value) shift_bit_set();           // 1 oder 0 entsprechend Wert setzen
                 else shift_bit_clr();

    shiftout_clkpuls();                         // ... Puls erzeugen
    mask= mask << 1;                             // Maske auf naechstes Bit setzen
  }
}

/* ----------------------------------------------------------
                           shiftout

     gibt das Byte in "Value" mit den in der globalen

                struct Variable shiftparam

     festgelegten Optionen seriell aus
   ---------------------------------------------------------- */
void shiftout(uint8_t value)
{
  if (shiftparam.msbfirst)
    shiftout_msbfirst(value);
  else
    shiftout_lsbfirst(value);
  shift_bit_clr();
}

/* ----------------------------------------------------------
                         shiftout_init

     setzt die an der seriellen Ausgabe beteiligten Portpins
     als Ausgang und legt in der globalen

                 struct Variable shiftparam

     die Schiebeoptionen fest.
   ---------------------------------------------------------- */
void shiftout_init(void)
{
  shift_clk_init();
  shift_bit_init();

  // Taktleitung ist im inaktiven Zustand (1 = low, 0 = high)
  shiftparam.clkpol= clkpol_default;

  // Anzahl Wiederholungen von _delay_us(1) waehrend des Taktpulses
  shiftparam.clkcycles = clkcycles_default;

  // Festlegen der Schieberichtung (1= MSB zuerst, 0 = LSB zuerst)
  shiftparam.msbfirst = msbfirst_default;

  if (shiftparam.clkpol)
    shift_clk_clr();
  else
    shift_clk_set();
}
