/* --------------------------------------------------------
                         softuart.c

     Softwaremodul fuer eine software realisierte serielle
     Schnittstelle.

     MCU:              ATmega8, ATmegaxx8, ATtinyx5, ATtinyx4

     Timerbenutzung           : TIMER0

     Originalcode von         : Colin Gittins
     AVR-Adaption             : Martin Thomas
     Modifikation 2015..2018  : R. Seelig

     Original-Anmerkungen und Copyrights siehe Dateiende

     13.11.2018

     Pinbelegung fuer TxD und RxD, sowie Baudrate siehe
     softuart.h
   --------------------------------------------------------*/


#include "softuart.h"

#define SU_TRUE    1
#define SU_FALSE   0

// startbit and stopbit parsed internally (see ISR)
#define RX_NUM_OF_BITS (8)

volatile static char           inbuf[uartsw_IN_BUF_SIZE];
volatile static unsigned char  qin;
static          unsigned char  qout;
volatile static unsigned char  flag_rx_off;
volatile static unsigned char  flag_rx_ready;

// 1 Startbit, 8 Databits, 1 Stopbit = 10 Bits/Frame
#define TX_NUM_OF_BITS (10)

volatile static unsigned char  flag_tx_busy;
volatile static unsigned char  timer_tx_ctr;
volatile static unsigned char  bits_left_in_tx;
volatile static unsigned short internal_tx_buffer;        // ! mt: was type uchar - this was wrong

#define set_tx_pin_high()      ( uartsw_TXPORT |=  ( 1 << uartsw_TXBIT ) )
#define set_tx_pin_low()       ( uartsw_TXPORT &= ~( 1 << uartsw_TXBIT ) )
#define get_rx_pin_status()    ( uartsw_RXPIN  &   ( 1 << uartsw_RXBIT ) )


/* ---------------------------------------------------------------
                            DEFINITONS
   --------------------------------------------------------------- */

ISR(uartsw_T_COMP_LABEL)
{
  static unsigned char flag_rx_waiting_for_stop_bit = SU_FALSE;
  static unsigned char rx_mask;

  static unsigned char timer_rx_ctr;
  static unsigned char bits_left_in_rx;
  static unsigned char internal_rx_buffer;

  unsigned char start_bit, flag_in;
  unsigned char tmp;

  // Transmitter Section

  if ( flag_tx_busy == SU_TRUE )
  {
    tmp = timer_tx_ctr;
    if ( --tmp == 0 )                           // if ( --timer_tx_ctr <= 0 )
    {
      if ( internal_tx_buffer & 0x01 )
      {
        set_tx_pin_high();
      }
      else
      {
        set_tx_pin_low();
      }
      internal_tx_buffer >>= 1;
      tmp = 3;                                  // timer_tx_ctr = 3;
      if ( --bits_left_in_tx == 0 )
      {
        flag_tx_busy = SU_FALSE;
      }
    }
    timer_tx_ctr = tmp;
  }

  // Receiver Section

  if ( flag_rx_off == SU_FALSE )
  {
    if ( flag_rx_waiting_for_stop_bit )
    {
      if ( --timer_rx_ctr == 0 )
      {
        flag_rx_waiting_for_stop_bit = SU_FALSE;
        flag_rx_ready = SU_FALSE;
        inbuf[qin] = internal_rx_buffer;
        if ( ++qin >= uartsw_IN_BUF_SIZE )      // overflow - reset inbuf-index
        {
          qin = 0;
        }
      }
    }
    else                                        // rx_test_busy
    {
      if ( flag_rx_ready == SU_FALSE )
      {
        start_bit = get_rx_pin_status();

        // test for start bit

        if ( start_bit == 0 )
        {
          flag_rx_ready = SU_TRUE;
          internal_rx_buffer = 0;
          timer_rx_ctr = 4;
          bits_left_in_rx = RX_NUM_OF_BITS;
          rx_mask = 1;
        }
      }
      else                                      // rx_busy
      {
        tmp = timer_rx_ctr;
        if ( --tmp == 0 )                       // if ( --timer_rx_ctr == 0 ) {
        {
                                                // rcv
          tmp = 3;
          flag_in = get_rx_pin_status();
          if ( flag_in )
          {
            internal_rx_buffer |= rx_mask;
          }
          rx_mask <<= 1;
          if ( --bits_left_in_rx == 0 )
          {
            flag_rx_waiting_for_stop_bit = SU_TRUE;
          }
        }
        timer_rx_ctr = tmp;
      }
    }
  }
}

// ---------------------------------------------------------------


static void io_init(void)
{

  uartsw_TXDDR |=  ( 1 << uartsw_TXBIT );       // TX-Pin as output
  uartsw_RXDDR &= ~( 1 << uartsw_RXBIT );       // RX-Pin as input

}

// ---------------------------------------------------------------

static void timer_init(void)
{
  unsigned char sreg_tmp;

   sreg_tmp = SREG;
   cli();

   uartsw_T_COMP_REG = uartsw_TIMERTOP;         // set top

   uartsw_T_CONTR_REGA = uartsw_CTC_MASKA | uartsw_PRESC_MASKA;
   uartsw_T_CONTR_REGB = uartsw_CTC_MASKB | uartsw_PRESC_MASKB;

   uartsw_T_INTCTL_REG |= uartsw_CMPINT_EN_MASK;

   uartsw_T_CNT_REG = 0;                        // reset counter */

   SREG = sreg_tmp;
}

// ---------------------------------------------------------------

void uartsw_init( void )
{
  flag_tx_busy  = SU_FALSE;
  flag_rx_ready = SU_FALSE;
  flag_rx_off   = SU_FALSE;

  set_tx_pin_high();                            // mt: set to high to avoid garbage on init

  io_init();
  timer_init();
  uartsw_turn_rx_on();                          // redundant - on by default */
  sei();
}

// ---------------------------------------------------------------

static void idle(void)
{
                                                // timeout handling goes here
                                                // - but there is a "uartsw_ischar" in this code...
                                                // add watchdog-reset here if needed
}

// ---------------------------------------------------------------

void uartsw_turn_rx_on( void )
{
  flag_rx_off = SU_FALSE;
}

// ---------------------------------------------------------------

void uartsw_turn_rx_off( void )
{
  flag_rx_off = SU_TRUE;
}

// ---------------------------------------------------------------

char uartsw_getchar( void )
{
  char ch;

  while ( qout == qin )
  {
    idle();
  }
  ch = inbuf[qout];
  if ( ++qout >= uartsw_IN_BUF_SIZE )
  {
    qout = 0;
  }
  return( ch );
}

// ---------------------------------------------------------------

unsigned char uartsw_ischar( void )
{
  return( qin != qout );
}

// ---------------------------------------------------------------

void uartsw_flush_input_buffer( void )
{
  qin  = 0;
  qout = 0;
}

// ---------------------------------------------------------------

unsigned char uartsw_transmit_busy( void )
{
  return ( flag_tx_busy == SU_TRUE ) ? 1 : 0;
}

// ---------------------------------------------------------------

void uartsw_putchar( const char ch )
{
  while ( flag_tx_busy == SU_TRUE )
  {
    ;                                           // wait for transmitter ready
                                                // add watchdog-reset here if needed;
  }

  // invoke_UART_transmit

  timer_tx_ctr = 3;
  bits_left_in_tx = TX_NUM_OF_BITS;
  internal_tx_buffer = ( ch << 1 ) | 0x200;
  flag_tx_busy = SU_TRUE;
}

// ---------------------------------------------------------------

void uartsw_putramstring(char *p )
{
  while ( *p )
  {
     uartsw_putchar( *p++ );
  }
}

// ---------------------------------------------------------------

void uartsw_putromstring(const unsigned char *dataPtr )
{
  char c;

  while (( c = pgm_read_byte( dataPtr++ )))
  {
     uartsw_putchar(c);
  }
}

// ---------------------------------------------------------------

void uartsw_crlf(void)
{
  uartsw_putchar(13);
  uartsw_putchar(10);
}

/* --------------------------------------------------
  Zeichen ueber einen Stream (bspw. ueber
  ScanF einlesen.
 -------------------------------------------------- */

int uartsw_filein( FILE *stream)
{
return uartsw_getchar();
}

/* --------------------------------------------------
  Zeichen ueber einen Stream (bspw. ueber
  PrintF versenden.
 -------------------------------------------------- */

int uartsw_fileout(char ch, FILE *stream)
{
  uartsw_putchar(ch);
}

/* --------------------------------------------------
    "Dateivariable" ueber den per Umleitung des
    Standartstreams (stdio) auf der seriellen
    Schnittstelle gesendet wird.
   -------------------------------------------------- */

// static FILE uartsw_tio = FDEV_SETUP_STREAM(uartsw_fileout,uartsw_filein,_FDEV_SETUP_RW);


// -----------------------------------------------------------------------------------------
//                                Original-Copyrights
// -----------------------------------------------------------------------------------------

// AVR-port of the generic software uart written in C
//
// Generic code from
// Colin Gittins, Software Engineer, Halliburton Energy Services
// (has been available from iar.com web-site -> application notes)
//
// Adapted to AVR using avr-gcc and avr-libc
// by Martin Thomas, Kaiserslautern, Germany
// <eversmith@heizung-thomas.de>
// http://www.siwawi.arubi.uni-kl.de/avr_projects
//
// AVR-port Version 0.4  10/2010
//
// ---------------------------------------------------------------------
//
// Remarks from Colin Gittins:
//
// Generic software uart written in C, requiring a timer set to 3 times
// the baud rate, and two software read/write pins for the receive and
// transmit functions.
//
// * Received characters are buffered
// * putchar(), getchar(), kbhit() and flush_input_buffer() are available
// * There is a facility for background processing while waiting for input
// The baud rate can be configured by changing the BAUD_RATE macro as
// follows:
//
// #define BAUD_RATE  19200.0
//
// The function init_uart() must be called before any comms can take place
//
// Interface routines required:
// 1. get_rx_pin_status()
//    Returns 0 or 1 dependent on whether the receive pin is high or low.
// 2. set_tx_pin_high()
//    Sets the transmit pin to the high state.
// 3. set_tx_pin_low()
//    Sets the transmit pin to the low state.
// 4. idle()
//    Background functions to execute while waiting for input.
// 5. timer_set( BAUD_RATE )
//    Sets the timer to 3 times the baud rate.
// 6. set_timer_interrupt( timer_isr )
//    Enables the timer interrupt.
//
// Functions provided:
// 1. void flush_input_buffer( void )
//    Clears the contents of the input buffer.
// 2. char kbhit( void )
//    Tests whether an input character has been received.
// 3. char getchar( void )
//    Reads a character from the input buffer, waiting if necessary.
// 4. void turn_rx_on( void )
//    Turns on the receive function.
// 5. void turn_rx_off( void )
//    Turns off the receive function.
// 6. void putchar( char )
//    Writes a character to the serial port.
//
// ---------------------------------------------------------------------

/*
Remarks by Martin Thomas (avr-gcc/avr-libc):
V0.1 (2/2005)
- stdio.h not used
- AVR-Timer in CTC-Mode ("manual" reload may not be accurate enough)
- Global Interrupt Flag has to be enabled (see demo-application)
- Interface timer_set and set_timer_interrupt not used here
- internal_tx_buffer was defined as unsigned char - thas could not
  work since more than 8 bits are needed, changed to unsigned short
- some variables moved from "global scope" into ISR function-scope
- GPIO initialisation included
- Added functions for string-output inspired by P. Fleury's AVR UART-lib.
V0.2 (3/2007)
- adjusted number of RX-bits
- adapted to avr-libc ISR-macro (replaces SIGNAL)
- disable interrupts during timer-init
- used unsigned char (uint8_t) where apropriate
- removed "magic" char checking (0xc2)
- added uartsw_can_transmit()
- Makefile based on template from WinAVR 1/2007
- reformated
- extended demo-application to show various methods to
  send a string from flash and RAM
- demonstrate usage of avr-libc's stdio in demo-applcation
- tested with ATmega644 @ 3,6864MHz system-clock using
  avr-gcc 4.1.1/avr-libc 1.4.5 (WinAVR 1/2007)
V0.3 (4/2007)
- better configuration options in uartsw.h.
  ->should be easier to adapt to different AVRs
- tested with ATtiny85 @ 1MHz (internal RC) with 2400bps
- AVR-Studio Project-File
V0.4 (10/2010)
- added options for ATmega164P, ATmega32P, ATmega64P
- changed some variable-types from char to unsigned char
- changed some comparisons from <= to ==
- small optimization in ISR for RX with temporary variable
- minor modifications in comments and formating
- added compiler options -fno-inline-small-functions, -Wl,--relax
- renamed flag_tx_ready to flag_tx_busy
- replaced uartsw_can_transmit() by uartsw_transmit_busy()
- tested with ATmega324PV @ 1MHz internal RC and 2400bps
  (options for ATtiny25/45/85 still available)
- added 3BSD license
- removed redundant zero-init in declaration of qin and qout
*/

/* Copyright (c) 2003, Colin Gittins
   Copyright (c) 2005, 2007, 2010, Martin Thomas
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

   * Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.

   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in
     the documentation and/or other materials provided with the
     distribution.

   * Neither the name of the copyright holders nor the names of
     contributors may be used to endorse or promote products derived
     from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. */
