/* ------------------------------------------------------------
                         t2313_n5110.c

   Softwaremodul fuer die Anbindung eines s/w Nokia LCD an
   einen ATtiny. Abgespeckte LCD-Bibliothek, keine Grafik-
   routinen

   MCU  : ATtiny2313
   Takt : 8 MHz

   28.01.2020 R. Seelig
   ---------------------------------------------------------- */

/*
                            ATTINY2313

                        Anschlusspins IC

                        +-------------+
          /reset - PA2  |  1       20 |  Vcc
             RxD - PD0  |  2       19 |  PB7 - UCSK - SCL - PCINT7
             TxD - PD1  |  3   A   18 |  PB6 - MISO - DO - PCINT6
           XTAL2 - PA1  |  4   T   17 |  PB5 - MOSI - DI - SDA - PCINT5
           XTAL1 - PA0  |  5   t   16 |  PB4 - OC1B - PCINT4
CKOUT - XCK - INT0 PD2  |  6   i   15 |  PB3 - OC1A - PCINT3
            INT1 - PD3  |  7   n   14 |  PB2 - OC0A - PCINT2
              T0 - PD4  |  8   y   13 |  PB1 - AIN1 - PCINT1
       OC0B - T1 - PD5  |  9       12 |  PB0 - AIN0 - PCINT0
                   GND  | 10       11 |  PD6 - ICP
                        +-------------+

Hinweis:

    bei Verwendung von USI als Hardware SPI ist der sendende Ausgang
    D0. Das heisst: PB6 entspricht hier dann MOSI (im Gegensatz zur
    Verwendung von ISP bei der der Anschluss PB6 MISO ist) !!!!


    Wiring:

    ATTiny2313             Display
    ------------------------------

       PB7 (19)   ---       CLK (5)
       PB6 (18)   ---       DIN (4)
       PB4 (16)   ---       DC  (3)
       PB3 (15)   ---       CE  (2)
       PB2 (14)   ---       RST (1)

*/
#include "t2313_n5110.h"

/*

  Die Prototypen aus t2313_n5110.h

  void spi_init(void);
  unsigned char spi_out(uint8_t data);
  void wrcmd(uint8_t command);                            // sende ein Kommando
  void lcd_init();                                        // initialisiere das Display
  void wrdata(uint8_t dat);                               // sende ein Datum
  void clrscr();                                          // loesche das Display
  void gotoxy(char x,char y);                             // positioniere Ausgabeposition
  void lcd_putchar_d(char ch);                            // setze Zeichen auf das Display
  void putramstring(char *c);                             // schreibe String aus dem RAM
  void putromstring(const unsigned char *dataPtr);        // dto. ROM

  #define prints(tx)     (putromstring(PSTR(tx)))         // Anzeige String aus Flashrom: prints("Text");
  #define printa(tx)     (putramstring(tx))               // Anzeige eines Strings der in einem Array im RAM liegt
*/

char wherex= 0;
char wherey= 0;
char invchar= 0;               // = 1 fuer inversive Textausgabe

/* -------------------- SPI Interface -------------------------- */

/* -------------------------------------------------------------
   SPI_INIT

     Anschlusspins des SPI-Interface konfigurieren
   ------------------------------------------------------------- */
void spi_init(void)
{
  SPI_DDR |= (1 << SPI_MOSI_PIN) | (1 << SPI_CLK_PIN) | (1 << SPI_SS_PIN);

  SPI_DDR &= ~(1 << SPI_MISO_PIN);
  SPI_PORT |= (1 << SPI_MISO_PIN);
}


#if (hardware_spi == 1)

  /* -------------------------------------------------------------
     SPI_OUT

        Byte ueber USI Hardware senden / empfangen
        data ==> zu sendendes Datum
     ------------------------------------------------------------- */
  unsigned char spi_out(uint8_t data)
  {
    volatile uint8_t r;

    USIDR = data;
    USISR = (1 << USIOIF);

    while ( (USISR & (1 << USIOIF)) == 0 )
    {
      USICR = (1 << USIWM0) | (1 << USICS1) | (1 << USICLK) | (1 << USITC);
    }
    return USIDR;
  }
#endif

#if (hardware_spi == 0)

  /* -------------------------------------------------------------
     SPI_OUT

        Byte ueber Software SPI senden / empfangen
        data ==> zu sendendes Datum
     ------------------------------------------------------------- */
  unsigned char spi_out(uint8_t data)
  {
    char a;

    for (a=0;a<8;a++)
    {
      if((data & 0x80)> 0) { LCD_PORT |= (1 << SPI_MOSI_PIN); }
                      else { LCD_PORT &= ~(1 << SPI_MOSI_PIN); }

      LCD_PORT |= (1 << SPI_CLK_PIN);                           // Taktleitung auf 1
      LCD_PORT &= ~(1 << SPI_CLK_PIN);                          // und wieder auf 0

      data <<= 1;
    }

  }

#endif


/* -------------------------------------------------------------
   WRCMD

   sendet Kommando via SPI an das LCD
   ------------------------------------------------------------- */
void wrcmd(uint8_t cmd)
{
  LCD_PORT &= ~(1 << LCD_DC_PIN);                    // C/D = 0 Kommandomodus
  LCD_PORT &= ~(1 << LCD_CE_PIN);                    // Display Controller enable
  spi_out(cmd);                                      // senden
  LCD_PORT |= (1 << LCD_CE_PIN );                    // Disable Controller
}

/* -------------------------------------------------------------
   WRDATA

   sendet Datum via SPI an das LCD
   ------------------------------------------------------------- */
void wrdata(uint8_t data)
{
  LCD_PORT |= (1 << LCD_DC_PIN);                     // C/D = 1 Kommandomodus
  LCD_PORT &= ~(1 << LCD_CE_PIN);                    // Display Controller enable
  spi_out(data);                                     // senden
  LCD_PORT |= (1 <<  LCD_CE_PIN );                   // Disable Controller
}

/* -------------------------------------------------------------
   LCD_INIT

   initialisiert das Display
   ------------------------------------------------------------- */
void lcd_init()
{
  LCD_DDR |= (1 << LCD_RST_PIN) | (1 << LCD_DC_PIN);     // Set LCD Output pins
  LCD_PORT &= ~(1 << LCD_RST_PIN);                       // Resets LCD controler
  _delay_ms(1);
  LCD_PORT |= (1 << LCD_RST_PIN);                        // Set LCD CE = 1 (Disabled)
  LCD_PORT |= (1 << LCD_CE_PIN);

  spi_init();

  // LCD Controller Kommandos  (eigentliches initialisieren)


  wrcmd(0x21);                            // Erweiterter Kommandomodus
  wrcmd(0x09);                            // Set int. HV Generator (ca. 7V an Pin7)
  wrcmd(0xff);                            // VOP max
  wrcmd(0x16);                            // BIAS = 2
  wrcmd(0x06);                            // Temp. Koeffizient = 2
  wrcmd(0x20);                            // Standart Kommandomodus
  wrcmd(0x0c);                            // normale Ausgabe (normal = 0Ch, invertiert = 0Dh)


// Alternative mit geaenderter Initialisierung  Displays
/*
  wrcmd(0x21);                            // Erweiterter Kommandomodus
  wrcmd(0x09);                            // interne Ladungspumpe (ca. 7V an Pin7)
  wrcmd(0xc8);                            // Temp Koefficent
  wrcmd(0x12);                            // BIAS = 0x10
  wrcmd(0x06);                            // Temp
  wrcmd(0x20);                            // Standart Kommandomodus
  wrcmd(0x0c);                            // normale Ausgabe (normal = 0Ch, invertiert = 0Dh)
*/
  clrscr();
}


/* -----------------------------------------------------
   CLRSCR

   loescht den Displayinhalt
   ----------------------------------------------------- */
void clrscr()
{
  int  i=0;

  wrcmd(0x80);             // Anfangsadresse des Displays
  wrcmd(0x40);
  for(i=1;i<(LCD_REAL_X_RES * LCD_REAL_Y_RES/8);i++) { wrdata(0x00); }
  gotoxy(0,0);
}

/* -----------------------------------------------------
   GOTOXY

   positioniert die Textausgabeposition auf X/Y
   ----------------------------------------------------- */
void gotoxy(char x,char y)
{
 wrcmd(0x80+(x*6));
 wrcmd(0x40+y);
 wherex= x; wherey= y;
}

/* -----------------------------------------------------
   PUTCHAR_D

   gibt ein Zeichen auf dem Display aus
   (die Steuerkommandos \n und \r fuer <printf> sind
   implementiert)
   ----------------------------------------------------- */
void lcd_putchar_d(char ch)
{
  int b,rb;

  if (ch== 13)
  {
    gotoxy(0,wherey);
    return;
  }
  if (ch== 10)
  {
    wherey++;
    gotoxy(wherex,wherey);
    return;
  }

  if (ch== 8)
  {
    if ((wherex> 0))
    {
      wherex--;
      gotoxy(wherex,wherey);
      for (b= 0;b<6;b++)
      {
        if (invchar) {wrdata(0xff);} else {wrdata(0);}
      }
      gotoxy(wherex,wherey);
    }
    return;
  }

  if ((ch<0x20)||(ch>lastascii)) ch = 92;               // ASCII Zeichen umrechnen

  // Kopiere Daten eines Zeichens aus dem Zeichenarray in den LCD-Screenspeicher

  for (b= 0;b<5;b++)
  {
    rb= pgm_read_byte(&(fonttab[ch-32][b]));
    if (invchar) {rb= ~rb;}
    wrdata(rb);
  }
  if (invchar) {wrdata(0xff);} else {wrdata(0);}
  wherex++;
  if (wherex> 15)
  {
    wherex= 0;
    wherey++;
  }
  gotoxy(wherex,wherey);
}

/* ---------------------------------------------------
   PUTRAMSTRING
   gibt einen Text aus dem RAM an der aktuellen Position aus
   ---------------------------------------------------*/
void putramstring(char *c)                              // Uebergabe eines Zeigers (auf einen String)
{
  while (*c)
  {
    lcd_putchar_d(*c++);
  }
}

/* ---------------------------------------------------
   PUTROMSTRING
   gibt einen Text aus dem ROM an der aktuellen Position aus
   ---------------------------------------------------*/
void putromstring(const unsigned char *dataPtr)
{
  unsigned char c;

  for (c=pgm_read_byte(dataPtr); c; ++dataPtr, c=pgm_read_byte(dataPtr)) lcd_putchar_d(c);
}

