/* -------------------------------------------------------
                         tetris128.c

                     YetAnotherTetrisGame

     Tetris Game fuer ATmega mit 128x128 TFT SPI Display

     MCU   :  ATmega168 / ATmega328
     Takt  :  16 MHz

     ATMega168 / 328p koennen auch ueber Arduino
     Bootloader geflasht werden. Wenn der blanke Chip
     mittels Programmer geflasht wird sind folgende
     Fuses einzustellen:

     MCU             LO   |   HI   |   EXT
     -------------------------------------
     ATmega328p      CF       D5       F9
     ATmega168       CF       D5       F9


     28.01.2020  R. Seelig
   ------------------------------------------------------ */

#include <stdint.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

#include <util/delay.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>

#include "avr_gpio.h"
#include "my_printf.h"
#include "tftdisplay.h"
#include "gfx_pictures.h"
#include "my_printf.h"


#if defined __AVR_ATmega328P__ || __AVR_ATmega168__

  #if defined __AVR_ATmega328P__


    #define big_logo        1
    #include "tetrislogo4.h"
    #include "tetleft.h"

  #endif

  #if defined __AVR_ATmega168__

    #define big_logo        0

    #include "tetrislogo3_cga.h"
    #include "tetleft_cga.h"

  #endif

#endif

#define delay        _delay_ms
#define printf       my_printf

#define intbit(wert,nr)    ((wert) & (1<<nr))
#define setintbit(wert,nr) (wert |= (1<<nr))
#define clrintbit(wert,nr) (wert &= ~(1<<nr))
                                              //               Arduino   28pol. Chip
#define tast1_init()    PD6_input_init()      // rechts            D6        12
#define tast2_init()    PD5_input_init()      // oben / drehen     D5        11
#define tast3_init()    PD4_input_init()      // links             D4         6
#define tast4_init()    PD3_input_init()      // unten             D3         5

#define is_tast1()      is_PD6()              // rechts
#define is_tast2()      is_PD5()              // oben
#define is_tast3()      is_PD4()              // links
#define is_tast4()      is_PD3()              // unten


#define  keyleft      3
#define  keyright     1
#define  keyup        2
#define  keydown      4

volatile char keys;

#define printf       my_printf


#define klotzgr     5
#define dryofs      24                                      // Y-Offset an der die Kloetzchen beginnen
#define drxofs      28                                      // dto. X-Koordinate
#define bmpofs      0                                       // X-Offset, ab der Images ausgegeben werden

#define gamrows     15
#define gamcol      11

#define scorepos    5,0
#define intime      60
#define outerloop   2


#define figanz   7
const uint16_t tetfigures[figanz][4] =
{
                                          //   o
   { 0xe400, 0x4c40, 0x4e00, 0x4640 },    //  ooo     Figur 0

                                          //  o
   { 0xe200, 0x44c0, 0x8e00, 0x6440 },    //  ooo     Figur 1

                                          //    o
   { 0xe800, 0xc440, 0x2e00, 0x4460 },    //  ooo     Figur 2

                                          //   oo
   { 0x6c00, 0x8c40, 0x6c00, 0x8c40 },    //  oo      Figur 3

                                          //  oo
   { 0xc600, 0x4c80, 0xc600, 0x4c80 },    //   oo     Figur 4

                                          //  oo
   { 0xcc00, 0xcc00, 0xcc00, 0xcc00 },    //  oo      Figur 5

                                          //
   { 0xf000, 0x4444, 0xf000, 0x4444 }     //  oooo    Figur 6
};

// Farbwerte der Kloetzchen: oberes Nibble  = Umrandungsfarbe,
//                           unteres Nibble = Fuellfarbe
const uint8_t colpair[figanz] =
{
  0x4e, 0x2a, 0x3b, 0x87, 0x5d, 0x19, 0x4c
};


uint8_t    tetfeld[gamrows][gamcol+2];       // Tetris Spielfeld, nimmt gamrows -  Reihen zu je gamcol Positionen + 2
                                             // Feldbegrenzer

uint8_t    aktfig, aktrot;
int8_t     figx, figy;
uint8_t    hb;                               // Hilfsbyte
uint8_t    loopcx;                           // Schleifenzaehler fuer Zeit und Tastenaktion
uint8_t    innerloop;
uint16_t   rndinit = 0;
int8_t     action;                           // Auszufuehrende Aktion waehrend des Spiels
uint16_t   hint;                             // Hilfsinteger
uint16_t   scorecx;
uint8_t    gameover;

/* --------------------------------------------------------
                         get16zufall

      generiert eine 16-Bit grosse Pseudozufallszahl die
      mittels mitgekoppeltem Schieberegister erreicht wird

    Uebergabe:
      startwert  : Ausgangspunkt im Zahlenbereich
      xormask    : Maske beim Schieben, hierdurch kann
                   ein Zahlenbereich eingeschraenkt werden

          xormask   Anzahl unterschiedlicher Zahlen
          -----------------------------------------
          0xb400                65535
          0x07c1                 2047
          0x0be0                 4095
  -------------------------------------------------------- */
uint16_t get16zufall(uint16_t startwert, uint16_t xormask)
{
  // Variable MUESSEN static sein, damit das Schieberegister
  // mit jedem Tick von der vorherigen Position aus weiter
  // schiebt
  static uint16_t start_state= 1;
  static uint16_t lfsr= 1;
  static int8_t first = 0;

  uint16_t lsb;

  if (first== 0)
  {
    first= 1;
    start_state= startwert;
    lfsr= start_state;
  }

  lsb = lfsr & 1;                        // niederwertigstes Bit des Schieberegisters
  lfsr = lfsr >> 1;                      // das Schieberegister, eine Stelle nach rechts
  if (lsb) { lfsr ^= xormask; }          // wenn LSB gesetzt, XOR-Togglemaske auf SR anwenden

  return lfsr;
}


/* --------------------------------------------------------
   my_putchar

   wird von my-printf / printf aufgerufen und hier muss
   eine Zeichenausgabefunktion angegeben sein, auf das
   printf dann schreibt !
   -------------------------------------------------------- */
void my_putchar(char ch)
{
  lcd_putchar(ch);
}


/* --------------------------------------------------------
                         getalltast

     fasst alle Tasten im unteren Nibble eines Bytes
     zusammen
   -------------------------------------------------------- */
uint8_t getalltast(void)
{
  uint8_t b;

  b= 0;
  if (!is_tast1()) b |= 1;
  if (!is_tast2()) b |= 2;
  if (!is_tast3()) b |= 4;
  if (!is_tast4()) b |= 8;

  return b;
}

/* --------------------------------------------------------
                Interrupt-Vector Timer1

       hier: wird durch timer1_init jede ms aufgerufen.
             erneuert hiermit jede Millisekunde den
             Tastenstatus in keys
   -------------------------------------------------------- */
ISR (TIMER1_COMPA_vect)
{
  keys= getalltast();
}

/* --------------------------------------------------------
                       timer1_init

     initialisiert Timer1 Interrupt fuer Aufruf jede
     Millisekund
   -------------------------------------------------------- */
void timer1_init(void)
{
  TCCR1B = 1<<WGM12 | 1<<CS10;
  OCR1A = F_CPU / 1000;                 // 1000 = Reloadwert fuer 16 MHz
  TCNT1 = 0;

  TIMSK1 = 1<<OCIE1A;
  sei();
}


/* --------------------------------------------------------
                        showscore

          zeigt Spielstand auf dem Display an
   -------------------------------------------------------- */
void showscore(void)
{
  gotoxy(scorepos);
  textcolor= rgbfromega(2);
  printf(" Score: ");
  textcolor= rgbfromega(10);
  printf("%d ",scorecx);
}

/* --------------------------------------------------------
                          drawklotz

     zeichnet ein einzelnes Kloetzchen (Bestandteil einer
     Tetrisfigur)

     Uebergabe:
          xpos, ypos  : "Kloetzchenkoordinate" im Spiele-
                        feld (ist hiermit wie ein sehr
                        grobes Pixel)
          fa          : farbe des Kloetzchens
          mode        : 1 => Kloetzchen wird gezeichnet
                        0 => wird geloescht
   -------------------------------------------------------- */
void drawklotz(int8_t xpos, int8_t ypos, uint8_t fa, char mode)
{
  int x,y;

  x= (xpos*(klotzgr+2))+drxofs;
  y= (ypos*(klotzgr+2))+dryofs;
  if (fa== 0x08)
  {
    line(x+(klotzgr / 2),y-1, x+(klotzgr/2), y+klotzgr, rgbfromega(8) );
  }
  else
  {
    if (mode)
    {
      fillrect(x+1,y+1, x+klotzgr-1, y+klotzgr-1, rgbfromega(fa & 0x0f));
//      rectangle(x,y, x+klotzgr, y+klotzgr, rgbfromega((fa >> 4) & 0x0f));
      rectangle(x,y, x+klotzgr, y+klotzgr, rgbfromega((fa) & 0x0f));
    }
    else
    {
      fillrect(x+1,y+1, x+klotzgr-1, y+klotzgr-1, rgbfromega(0));
      rectangle(x,y, x+klotzgr, y+klotzgr, rgbfromega(0));
    }
  }
}

/* --------------------------------------------------------
                           drawfig

     zeichnet eine Tetrisfigur im Spielfeld

     Uebergabe:
          xpos, ypos  : "Kloetzchenkoordinate" im Spiele-
                        feld (ist hiermit wie ein sehr
                        grobes Pixel)
          figbmp      : 16-Bit grosses Bitmuster der
                        Figur. Ein einzelnes Bit wird als
                        Kloetzchen gezeichnet
          fa          : farbe des Kloetzchens
          mode        : 1 => Kloetzchen wird gezeichnet
                        0 => wird geloescht
   -------------------------------------------------------- */
void drawfig(int8_t xpos, int8_t ypos, uint16_t figbmp, uint8_t fa, char mode)
{
  int8_t      x,y,i;

  for (i= 15; i>= 0; i--)
  {
    x= (15-i)%4;
    y= (15-i)/4;
    if (intbit(figbmp,i)) drawklotz(x+xpos, y+ypos, fa, mode);
  }
}

/* --------------------------------------------------------
                          readkey

     liefert den Wert einer gedrueckten Taste zurueck.
     "keys" wird vom Timerinterrupt jede Millisekunde
     aktualisiert
   -------------------------------------------------------- */
char readkey()
{
  if (keys & 0x01) return 1;           // Taste rechts  / keyright
  if (keys & 0x02) return 2;           // Taste oben    / keyup
  if (keys & 0x04) return 3;           // Taste links   / keyleft
  if (keys & 0x08) return 4;           // Taste runter  / keydown
  return 0;
}

/* --------------------------------------------------------
                      gamefeld_init

            initialisiert ein leeres Spielfeld
   -------------------------------------------------------- */
void gamefeld_init(void)
{
  uint8_t x, y;

  for (y= 0; y< gamrows; y++)
  {
    for (x= 1; x< (gamcol+1); x++)
    {
      tetfeld[y][x]= 0;
    }
    tetfeld[y][0]= 0xff;
    tetfeld[y][gamcol+1]= 0xff;
  }
  for (x= 0; x< (gamcol+2); x++)
  {
    tetfeld[gamrows-1][x]= 0xff;
  }
}

/* --------------------------------------------------------
                        drawfeld

     zeichnet das Spielfeld mit dem Inhalt von "tetfeld"
   -------------------------------------------------------- */
void drawfeld(void)
{
  uint8_t x, y, b;

  for (y= 0; y< gamrows-1; y++)
  {
    for (x= 0; x< (gamcol+2); x++)
    {
      b= tetfeld[y][x];
      if (b== 0xff)
      {
        drawklotz(x,y,0x08,1);
      }
      if (b== 0x00)
      {
        drawklotz(x,y,0x07,0);
      }
      if ((b>0) && (b != 0xff))
      {
        drawklotz(x,y,b+8,1);
      }
    }
  }
}

/* -------------------------------------------------
                        canposxy

     testet, ob an der angegebenen Position die
     uebergebene Figur positionierbar ist !
   ------------------------------------------------- */
uint8_t canposxy(uint8_t fx, uint8_t fy, uint16_t fig)
{
  uint8_t x,y,ind;

  ind= 15;
  for (y= fy; y< (fy+4); y++)
  {
    for (x= fx; x< (fx+4); x++)
    {
      if ((tetfeld[y][x]> 0) & (intbit(fig,ind)> 0))
      {
        return 0;
      }
      ind--;
    }
  }
  return 1;
}

/* -------------------------------------------------
                        copyfig
    kopiert die aktuelle Spielefigur in der
    aktuellenk Rotationslage ins Spielegamefeld
    mit der angegeben Figurnummer
   ------------------------------------------------- */
void copyfig(int8_t figx, int8_t figy, uint16_t fig, uint8_t nr)
{
  uint8_t x,y,ind;

  ind= 15;
  for (y= figy; y < (figy + 4); y++)
  {
    for (x= figx; x < (figx + 4); x++)
    {
      if (intbit(fig,ind)>0) { tetfeld[y][x]= nr; }
      ind--;
    }
  }
}

/* -------------------------------------------------
                         delrow

     loescht eine Reihe aus dem Spielfeld "tetfeld"
     und positioniert die darueber liegenden
     Kloetzchen um eine Position nach unten

     Uebergabe:
          nr :  zu loeschende Reihe
   ------------------------------------------------- */
void delrow(int8_t nr)
{
  int x,y;

  for (y= nr; y>= 0; y--)
  {
    for (x= 1; x< gamcol+1; x++)
    {
      tetfeld[y][x]= tetfeld[y-1][x];
    }
  }
  for (x= 1; x< gamcol+1; x++)
  {
    tetfeld[0][x]= 0x00;
  }
  scorecx++;
  showscore();
}

/* -------------------------------------------------
                   scangamefeld

      entfernt eine solchige bei gleichzeitiger
      Erhoehung des  Spielstands "scorecx"
   ------------------------------------------------- */
uint8_t scangamefeld(void)
//   Scant das Spielfeld auf vollstaendig ausgefuellte Reihen und

{
  int8_t x,y;
  int8_t kcnt;
  int8_t retval;

  retval= 0;
  for (y= gamrows-2; y>= 0 ; y--)
  {
    kcnt= 0;
    for (x= 1; x< gamcol+1; x++)
    {
      if (tetfeld[y][x]> 0) { kcnt++; }
    }
    if (kcnt== gamcol)
    {
      delrow(y);
      retval= 1;
    }
  }
  return retval;
}

/* -------------------------------------------------
                     checkdownrow

     Testet, ob die aktuelle Spielefigur abgelegt
     werden kann und positioniert diese, wenn nicht
     wird das globale Flag "gameover" gesetzt
   ------------------------------------------------- */
void checkdownrow(void)
{
  copyfig(figx,figy,tetfigures[aktfig][aktrot],aktfig+1);
  while(scangamefeld());
  drawfeld();

  aktfig= get16zufall(rndinit, 0x0be0) % 7;

  aktrot= 0; figx= 5; figy= 0;
  action= 0;
  if (canposxy(figx,figy,tetfigures[aktfig][aktrot]))
  {
    drawfig(figx,figy,tetfigures[aktfig][aktrot],aktfig+9,1);
  }
  else {gameover= 1;}
}


/* ----------------------------------------------------------------------
                                   MAIN
   ---------------------------------------------------------------------- */
int main(void)
{

  int     ch;
  uint8_t x,y;
  int8_t    endgame = 0;

  lcd_init();
  textsize= 0;
  setfont(0);
  lcd_orientation(3);
  lcd_enable();
  bkcolor= rgbfromega(0);
  textcolor= rgbfromega(7);

  tast1_init();
  tast2_init();
  tast3_init();
  tast4_init();

  timer1_init();

  delay(100);

  bkcolor= 0;
  clrscr();
  textcolor= rgbfromega(9);
  #if (big_logo == 1)
    bmp16_show(bmpofs,0,&tetrislogo[0], &logopal[0]);
  #else
    bmpcga_show(bmpofs,0,&tetrislogo[0], &logopal[0]);
  #endif

  do
  {
    ch= readkey();
    rndinit++;
    rndinit= rndinit % 2048;
  }while(ch== 0);


  while(1)
  {
    gamefeld_init();
    delay(300);

    while(endgame== 0)
    {
      bkcolor= 0;
      clrscr();
      fillrect(bmpofs+32,22,bmpofs+112,122,rgbfromvalue(0x10,0x10,0x10));
      #if (big_logo == 1)
        bmp16_show(bmpofs,8,&tetleft[0], &tetleftpal[0]);
      #else
        bmpcga_show(bmpofs,8,&tetleft[0], &logopal[0]);
      #endif
      textcolor= rgbfromega(7);
      fontnr= 2;
      outtextxy(119,20,1, "AVR - ATmega");
      fontnr= 0;

      scorecx= 0;
      while(scangamefeld());
      drawfeld();
      showscore();

      aktfig= get16zufall(rndinit, 0x0be0) % 7;

      aktrot= 0; figx= 5; figy= 0;
      action= 0;
      loopcx= outerloop;
      drawfig(figx,figy,tetfigures[aktfig][aktrot],aktfig+9,1);

      gameover= 0;
      while((gameover==0) && (endgame== 0))
      {
        switch (action)
        {
          case 1:                                                       // rotieren
            hb= aktrot;
            aktrot++;
            aktrot %= 4;
            hint= tetfigures[aktfig][aktrot];
            if (aktfig==6) {hint= 0xffff;}                              // Figur 7 (langer Stab) kann nur
                                                                        // gedreht werden, wenn alle Plaetze
                                                                        // unter ihm unbelegt sind !!!
            if (canposxy(figx,figy,hint))
            {
              drawfig(figx,figy,tetfigures[aktfig][hb],aktfig+9,0);
              drawfig(figx,figy,tetfigures[aktfig][aktrot],aktfig+9,1);
            }
            else
            {
              aktrot= hb;                                               // rotieren war nicht moeglich;
            }
            break;

          case 2:                                                       // rechts schieben
            if (canposxy(figx+1,figy,tetfigures[aktfig][aktrot]))
            {
              drawfig(figx,figy,tetfigures[aktfig][aktrot],aktfig+9,0);
              figx++;
              drawfig(figx,figy,tetfigures[aktfig][aktrot],aktfig+9,1);

            }
            break;

          case 3:                                                       // links schieben
            if (canposxy(figx-1,figy,tetfigures[aktfig][aktrot]))
            {
              drawfig(figx,figy,tetfigures[aktfig][aktrot],aktfig+9,0);
              figx--;
              drawfig(figx,figy,tetfigures[aktfig][aktrot],aktfig+9,1);

            }
            break;
          case 4:
            while (canposxy(figx,figy+1,tetfigures[aktfig][aktrot])) {figy++;}
            showscore();
            checkdownrow();
            break;
        }

        if (innerloop> 0)
        {
          for (hb= innerloop; hb> 0; hb--)
          {
            delay(1);
          }
        }

        action= 0;
        while ((action==0) & (loopcx>0))
        {
          loopcx--;
          innerloop= intime;
          while ((action==0) & (innerloop > 0))
          {
            ch= readkey();
            switch (ch)
            {
              case keyup    :   action= 1; break;
              case keyright :   action= 2; break;
              case keyleft  :   action= 3; break;
              case keydown  :   action= 4; break;
              default       :   break;
            }
            if (ch>0) delay(80);
            delay(3);
            innerloop--;
          }
          if ((action<4) & (action>0)) { innerloop= intime; }
        }
        if (loopcx== 0)
        {
          loopcx= outerloop;
          if (canposxy(figx,figy+1,tetfigures[aktfig][aktrot]))
          {
            drawfig(figx,figy,tetfigures[aktfig][aktrot],aktfig+9,0);
            figy++;
            drawfig(figx,figy,tetfigures[aktfig][aktrot],aktfig+9,1);

          }
          else
          {
            showscore();
            checkdownrow();
          }
        }
      }
      if (!endgame)
      {
        for (y= 0; y < gamrows-1; y++)
        {
          for (x= 1; x < gamcol+1; x++)
          {
            drawklotz(x, y, 5, 1);
            delay(10);
          }
        }
        textcolor= rgbfromega(15);
        fillrect(48,56, 96,88, rgbfromega(0));
        outtextxy(58,63,0, "Game");
        outtextxy(58,73,0, "over");
        while( !(readkey()) );
        gamefeld_init();
        endgame= 0;
      }
    }
  }
}

