/* ----------------------- usi_serial_demo2.c -----------------------

     Demoprogramm zur Verwendung des USI-Interfaces als
     UART - RS232 Schnittstelle

     Verwendet USI-Overflow, Pinchange0 und Timer0 Compare Match
     Interrupt


     MCU   : ATtiny44
     F_CPU : 8 MHz intern

     Fuses : fuer 8 MHz intern
             lo 0xe2
             hi 0xdf

     Fuses : fuer ext. Quarz >= 8 MHz
             lo 0xde
             hi 0xdf


     Quellen: AVR App-Note AVR307 und diverse Internetquellen

     TxD Anschluss: PA5 (DO)
     RxD Anschluss: PA6 (DI)

     17.08.2017 R. Seelig
   ------------------------------------------------------------------ */

/*
                                                 ATtiny44
                                        4 kByte Flash / 256 Byte SRAM

                                             Anschlusspins IC

                                              +-----------+
                                         Vcc  | 1      14 |  GND
                   PCINT8 - XTAL1 - CLKI PB0  | 2   A  13 |  PA0 - ADC0 - AREF - PCINT0
                        PCINT9 - XTAL2 - PB1  | 3   T  12 |  PA1 - ADC1 - AIN0 - PCINT1
                 PCINT11 - /reset - dW - PB3  | 4   t  11 |  PA2 - ADC2 - AIN1 - PCINT2
         PCINT10 - INT0 - OC0A - CKOUT - PB2  | 5   i  10 |  PA3 - ADC3 - T0 - PCINT3
            PCINT7 - ICP - OC0B - ADC7 - PA7  | 6   n   9 |  PA4 - ADC4 - USCK - SCL - T1 - PCINT4
PCINT6 - OC1A - SDA - MOSI - DI - ADC6 - PA6  | 7   y   8 |  PA5 - ADC5 - DO - MISO - OC1B PCINT5
                                              +-----------+

*/

#include <util/delay.h>
#include <avr/io.h>

#include "tx4_usiuart.h"
#include "my_printf.h"
#include "avr_gpio.h"

#define  printf               tiny_printf
#define  delay                _delay_ms

#define cnt_speed    500

#define led_init()     PA1_output_init()
#define led_clr()      PA1_set()
#define led_set()      PA1_clr()


/* -------------------------------------------------------------
                              tiny_pow

       berechnet value^n wobei n ein Integer sein muss
   ------------------------------------------------------------- */
float tiny_pow(int n, float value)
{
  float tmp;
  int   i;

  tmp= value;
  for (i= 0; i < n-1; i++)
  {
    tmp= tmp*value;
  }
  return tmp;
}

float tiny_sin(float value)
{

  float degree;
  float p3;
  float p5;
  float p7;
  float sinx;

  int   mflag= 0;

  while (value > 360.0f) value -= 360.0f;
  if (value > 180.0f)
  {
    mflag= - 1;
    value -= 180.0f;
  }

  if (value > 90.0f) value = 180.0f - value;

  degree= (value * M_PI) / 180.0f;

  p3 = tiny_pow(3, degree);
  p5  = tiny_pow(5, degree);
  p7  = tiny_pow(7, degree);

  sinx= (degree - (p3/6.0f) + (p5/120.0f) - (p7/5040.0f));

  if (mflag) sinx = sinx * (-1);
  return sinx;

}

void draw_sincurve(void)
{
  int i, x, x2;
  for (i= 0; i< 361; i += 10)
  {
    x2 = (tiny_sin(i) * 24) + 25;
    for (x= 0; x < x2; x++) uart_putchar(' ');
    printf("o\n\r");
    delay(40);
  }
}

/* --------------------------------------------------------
   my_putchar

   wird von my-printf / printf aufgerufen und hier muss
   eine Zeichenausgabefunktion angegeben sein, auf das
   printf dann schreibt !
   -------------------------------------------------------- */
void my_putchar(char ch)
{
  uart_putchar(ch);
}


int main(void)
{
  long        li;
  uint16_t    baudrate;

  uint16_t    counter = 0;
  uint16_t    ch;
  float       wuwert;
  int         wuwerti;

  printfkomma= 3;

  uart_init();

  led_init();

  printf("\n\r--------------------------------------");
  printf("\n\rAttiny44 running at 8 MHz\n\r");
  printf("8 kByte Flash;  1 KByte RAM\n\n\r");
  printf("Baudrate: 19200 bd\n\n\r");
  printf("20.04.2017  R. Seelig\n\r");
  printf("--------------------------------------\n\r");
  printf("Taste um Counter anzuhalten bzw. neu zu starten\n\n\r");

  while(uart_ischar()) {ch= uart_getchar(); my_putchar(ch); }  // eventuell eingegangene Zeichen alle loeschen

  counter= 0;
  while (1)
  {
    led_set();
    delay(cnt_speed);
    led_clr();

    wuwert= tiny_sin(counter);
    wuwerti= wuwert*1000;

    printf("  Counter: %xh = %dd | sin(%d)= %k \r", counter, counter, counter % 360, wuwerti);

    delay(cnt_speed);
    counter++;
    counter = counter % 3600;

    if (uart_ischar())
    {
      ch= uart_getchar();
      printf("\n\n\rGedrueckte Taste war: %c\n\r", ch );
      printf("Beliebige Taste fuer Counterstart...\n\n\r");
      while (!uart_ischar());
      ch= uart_getchar();
      draw_sincurve();

    }

  }
}
