/* ----------------------------------------------------------
                            pwmtest.c

     Versuche mit Pulsweitenmodulation auf Timer1 und Timer2


     MCU     : ATmegaxx8
     F_CPU   :

     06.01.2021    R. Seelig
   ---------------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <util/delay.h>
#include <avr/io.h>
#include <avr/pgmspace.h>

#include "avr_gpio.h"
#include "pwm.h"

#define printf    my_printf

#define puts(txt)       uart_putromstring(PSTR(txt))  // String aus Flashspeicher anzeigen

#define BAUDRATE   19200

/* --------------------------------------------------
                  uart_init

    Initialisierung der seriellen Schnittstelle,
    Baudrate siehe define
    Protokoll: 8 Daten-, 1 Stopbit
   -------------------------------------------------- */
void uart_init(void)
{
  uint16_t ubrr;

  // fuer Baudraten groesser 57600
  if (BAUDRATE> 57600)
  {
    ubrr= (F_CPU / 16 / (BAUDRATE >> 1));
    ubrr--;
    UCSR0A |= 1<<U2X0;                                  // Baudrate verdoppeln
  }
  else
  {
    ubrr= ( F_CPU / 16 / BAUDRATE-1);
  }

  UBRR0H = (uint8_t)(ubrr>>8);                          // Baudrate setzen
  UBRR0L = (uint8_t)ubrr;

  UCSR0B = (1<<RXEN0)|(1<<TXEN0);                       // Transmitter und Receiver enable
  UCSR0C = (3<<UCSZ00);                                 // 8 Datenbit, 1 Stopbit
}

/* --------------------------------------------------
                       uart_putchar

    Zeichen ueber URT Schnittstelle senden
   -------------------------------------------------- */

void uart_putchar(uint8_t ch)
{
  while (!( UCSR0A & (1<<UDRE0)));                      // warten bis Transmitterpuffer leer ist
  UDR0 = ch;                                            // Zeichen senden
}

uint8_t uart_getchar( void )
{
  while(!(UCSR0A & (1<<RXC0)));                         // warten bis Zeichen eintrifft

#if (echo_enable == 1)

  char ch;
  ch= UDR0;
  uart_putchar(ch);
  return ch;

#else

  return UDR0;

#endif
}


/* --------------------------------------------------
                  uart_putromstring

     gibt einen String aus dem ROM ueber die RS-232
     aus.

     Bsp.:

          uart_putromstring(PSTR("Hallo Welt")
   -------------------------------------------------- */
void uart_putromstring(const uint8_t *dataPtr)
{
  unsigned char c;

  for (c=pgm_read_byte(dataPtr); c; ++dataPtr, c=pgm_read_byte(dataPtr))
  {
    uart_putchar(c);
  }
}

/* ------------------------------------------------------------
                           put_u16int

     gibt einen 16-BitInteger dezimal aus. Ist Uebergabe
     "komma" != 0 wird ein "Kommapunkt" mit ausgegeben.

     Bsp.: 12345 wird bei komma==2 als 123.45 ausgegeben.
     (ermoeglicht Pseudofloatausgaben im Bereich)
   ------------------------------------------------------------ */
void put_u16int(uint16_t i, char komma)
{
  typedef enum boolean { FALSE, TRUE }bool_t;

  static uint16_t zz[]      = { 10000, 1000, 100, 10 };
  bool_t          not_first = FALSE;

  uint8_t   zi;
  int32_t   z, b;

  komma= 5-komma;

  if (!i)
  {
    uart_putchar('0');
  }
  else
  {

    for(zi = 0; zi < 4; zi++)
    {
      z = 0;
      b = 0;

      if  ((zi== komma) && komma)
      {
        if (!not_first) uart_putchar('0');
        uart_putchar('.');
        not_first= TRUE;
      }

      while(z + zz[zi] <= i)
      {
        b++;
        z += zz[zi];
      }
      if(b || not_first)
      {
        uart_putchar('0' + b);
        not_first = TRUE;
      }
      i -= z;
    }
    if (komma== 4) uart_putchar('.');
    uart_putchar('0' + i);
  }
}


/* ------------------------------------------------------------
                           read_u16int

     liest einen 16-Bit unsigned Integer auf der seriellen
     Schnittstelle ein

     Korrektur ist mit der Loeschtaste nach links moeglich.
   ------------------------------------------------------------ */
uint16_t read_u16int(void)
{
  uint16_t  sum = 0;
  uint8_t   zif;
  uint8_t   ch;

  do
  {
    ch= uart_getchar();
    if (ch== 0x0a) ch= 0x0d;

    // Ziffern auswerten
    if ((ch>= '0') && (ch<= '9'))
    {
      zif= ch-'0';

      if ((sum== 0) && (zif))        // erste Ziffer
      {
        sum= zif;
        uart_putchar(ch);
      }
      else
      if (sum < 6554)
      {
        if  (!( ((sum * 10) > 65520) && (zif > 5) ))
        {
          sum= (sum*10) + zif;
          uart_putchar(ch);
        }
      }
    }

    // letzte Eingabe loeschen
    if ((ch== 127) || (ch== 8))    // letzte Eingabe loeschen
    {
      if (sum)
      {
        sum /= 10;
        uart_putchar(8);
        uart_putchar(' ');
        uart_putchar(8);
      }
    }

  } while (ch != 0x0d);              // wiederholen bis Returnzeichen eintrifft
  return sum;
}



/* ---------------------------------------------------------------------------
                                    M A I N
   --------------------------------------------------------------------------- */
int main(void)
{
  uint16_t freq, duty;

  uart_init();

  puts("\n\r ----------------------------------");
  puts("\n\r     Rechteck-Frequenzgenerator");
  puts("\n\r     PWM-Demo");
  puts("\n\r     06.01.2021      R. Sellig");
  puts("\n\r ----------------------------------");
  puts("\n\r");

  pwmt1_setfreq(1000, 50);

  while(1)
  {
    puts("\n\r set frequency    : ");
    freq= read_u16int();
    puts("\n\r set duty cycle   : ");
    duty= read_u16int();
    freq= pwmt1_setfreq(freq, duty);
    puts("\n\r set frequency to : ");
    put_u16int(freq,0);
    puts("\n\r");
  }
}
